@@ -15,20 +15,42 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: copyin.c,v 1.13 1997/11/08 20:17:46 rob Exp $
  */
 
-# include <unistd.h>
-# include <sys/stat.h>
-# include <fcntl.h>
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_FCNTL_H
+#  include <fcntl.h>
+# else
+int open(const char *, int, ...);
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# else
+int dup(int);
+# endif
+
+# include <stdlib.h>
 # include <string.h>
+# include <errno.h>
 
 # include "hfs.h"
 # include "data.h"
 # include "copyin.h"
+# include "charset.h"
 # include "binhex.h"
 # include "crc.h"
 
-char *cpi_error = "no error";
+const char *cpi_error = "no error";
+
+extern int errno;
+
+# define ERROR(code, str)	(cpi_error = (str), errno = (code))
 
 # define MACB_BLOCKSZ	128
 
@@ -40,76 +62,127 @@ char *cpi_error = "no error";
 
 /* Copy routines =========================================================== */
 
+/*
+ * NAME:	fork->macb()
+ * DESCRIPTION:	copy a single fork for MacBinary II
+ */
+static
+int fork_macb(int ifile, hfsfile *ofile, unsigned long size)
+{
+  char buf[HFS_BLOCKSZ * 4];
+  long chunk, bytes;
+
+  while (size)
+    {
+      chunk = (size < sizeof(buf)) ?
+	(size + (MACB_BLOCKSZ - 1)) & ~(MACB_BLOCKSZ - 1) : sizeof(buf);
+
+      bytes = read(ifile, buf, chunk);
+      if (bytes == -1)
+	{
+	  ERROR(errno, "error reading data");
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "read incomplete chunk");
+	  return -1;
+	}
+
+      chunk = (size > bytes) ? bytes : size;
+
+      bytes = hfs_write(ofile, buf, chunk);
+      if (bytes == -1)
+	{
+	  ERROR(errno, hfs_error);
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
+	  return -1;
+	}
+
+      size -= chunk;
+    }
+
+  return 0;
+}
+
 /*
  * NAME:	do_macb()
  * DESCRIPTION:	perform copy using MacBinary II translation
  */
 static
-int do_macb(int ifile, hfsfile *ofile, long dsize, long rsize)
+int do_macb(int ifile, hfsfile *ofile,
+	    unsigned long dsize, unsigned long rsize)
 {
-  char buf[HFS_BLOCKSZ * 4];
-  long bytes, chunk;
-
-  cpi_error = "error reading data";
-
-  /* data fork */
-
-  while (dsize)
+  if (hfs_setfork(ofile, 0) == -1)
     {
-      if (dsize < sizeof(buf))
-	chunk = (dsize + (MACB_BLOCKSZ - 1)) & ~(MACB_BLOCKSZ - 1);
-      else
-	chunk = sizeof(buf);
-
-      bytes = read(ifile, buf, chunk);
-      if (bytes < chunk)
-	return -1;
-
-      if (dsize > bytes)
-	chunk = bytes;
-      else
-	chunk = dsize;
-
-      if (hfs_write(ofile, buf, chunk) < 0)
-	{
-	  cpi_error = hfs_error;
-	  return -1;
-	}
-
-      dsize -= chunk;
-    }
-
-  /* resource fork */
-
-  if (hfs_setfork(ofile, 1) < 0)
-    {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  while (rsize)
+  if (fork_macb(ifile, ofile, dsize) == -1)
+    return -1;
+
+  if (hfs_setfork(ofile, 1) == -1)
     {
-      if (rsize < sizeof(buf))
-	chunk = (rsize + (MACB_BLOCKSZ - 1)) & ~(MACB_BLOCKSZ - 1);
-      else
-	chunk = sizeof(buf);
+      ERROR(errno, hfs_error);
+      return -1;
+    }
 
-      bytes = read(ifile, buf, chunk);
-      if (bytes < chunk)
-	return -1;
+  if (fork_macb(ifile, ofile, rsize) == -1)
+    return -1;
 
-      if (rsize > bytes)
-	chunk = bytes;
-      else
-	chunk = rsize;
+  return 0;
+}
 
-      if (hfs_write(ofile, buf, chunk) < 0)
+/*
+ * NAME:	fork->binh()
+ * DESCRIPTION:	copy a single fork for BinHex
+ */
+static
+int fork_binh(hfsfile *ofile, unsigned long size)
+{
+  char buf[HFS_BLOCKSZ * 4];
+  long chunk, bytes;
+
+  while (size)
+    {
+      chunk = (size > sizeof(buf)) ? sizeof(buf) : size;
+
+      bytes = bh_read(buf, chunk);
+      if (bytes == -1)
 	{
-	  cpi_error = hfs_error;
+	  ERROR(errno, bh_error);
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "read incomplete chunk");
 	  return -1;
 	}
 
-      rsize -= chunk;
+      bytes = hfs_write(ofile, buf, chunk);
+      if (bytes == -1)
+	{
+	  ERROR(errno, hfs_error);
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
+	  return -1;
+	}
+
+      size -= chunk;
+    }
+
+  if (bh_readcrc() == -1)
+    {
+      ERROR(errno, bh_error);
+      return -1;
     }
 
   return 0;
@@ -120,76 +193,25 @@ int do_macb(int ifile, hfsfile *ofile, long dsize, long rsize)
  * DESCRIPTION:	perform copy using BinHex translation
  */
 static
-int do_binh(hfsfile *ofile, long dsize, long rsize)
+int do_binh(hfsfile *ofile, unsigned long dsize, unsigned long rsize)
 {
-  char buf[HFS_BLOCKSZ * 4];
-  long bytes;
-
-  /* data fork */
-
-  while (dsize)
+  if (hfs_setfork(ofile, 0) == -1)
     {
-      if (dsize > sizeof(buf))
-	bytes = sizeof(buf);
-      else
-	bytes = dsize;
-
-      if (bh_read(buf, bytes) < bytes)
-	{
-	  cpi_error = bh_error;
-	  return -1;
-	}
-
-      if (hfs_write(ofile, buf, bytes) < 0)
-	{
-	  cpi_error = hfs_error;
-	  return -1;
-	}
-
-      dsize -= bytes;
-    }
-
-  if (bh_readcrc() < 0)
-    {
-      cpi_error = bh_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  /* resource fork */
+  if (fork_binh(ofile, dsize) == -1)
+    return -1;
 
-  if (hfs_setfork(ofile, 1) < 0)
+  if (hfs_setfork(ofile, 1) == -1)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  while (rsize)
-    {
-      if (rsize > sizeof(buf))
-	bytes = sizeof(buf);
-      else
-	bytes = rsize;
-
-      if (bh_read(buf, bytes) < bytes)
-	{
-	  cpi_error = bh_error;
-	  return -1;
-	}
-
-      if (hfs_write(ofile, buf, bytes) < 0)
-	{
-	  cpi_error = hfs_error;
-	  return -1;
-	}
-
-      rsize -= bytes;
-    }
-
-  if (bh_readcrc() < 0)
-    {
-      cpi_error = bh_error;
-      return -1;
-    }
+  if (fork_binh(ofile, rsize) == -1)
+    return -1;
 
   return 0;
 }
@@ -201,29 +223,46 @@ int do_binh(hfsfile *ofile, long dsize, long rsize)
 static
 int do_text(int ifile, hfsfile *ofile)
 {
-  char buf[HFS_BLOCKSZ * 4], *ptr;
-  long bytes;
+  unsigned char buf[HFS_BLOCKSZ * 4], *ptr;
+  long chunk, bytes;
+  int len;
 
   while (1)
     {
-      bytes = read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      chunk = read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
 	{
-	  cpi_error = "error reading source file";
+	  ERROR(errno, "error reading source file");
 	  return -1;
 	}
-      else if (bytes == 0)
+      else if (chunk == 0)
 	break;
 
-      for (ptr = buf; ptr < buf + bytes; ++ptr)
+      for (ptr = buf; ptr < buf + chunk; ++ptr)
 	{
 	  if (*ptr == '\n')
 	    *ptr = '\r';
 	}
 
-      if (hfs_write(ofile, buf, bytes) < 0)
+      len = chunk;
+      ptr = cs_macroman(buf, &len);
+      if (ptr == 0)
 	{
-	  cpi_error = hfs_error;
+	  ERROR(ENOMEM, 0);
+	  return -1;
+	}
+
+      bytes = hfs_write(ofile, ptr, len);
+      free(ptr);
+
+      if (bytes == -1)
+	{
+	  ERROR(errno, hfs_error);
+	  return -1;
+	}
+      else if (bytes != len)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
 	  return -1;
 	}
     }
@@ -239,22 +278,28 @@ static
 int do_raw(int ifile, hfsfile *ofile)
 {
   char buf[HFS_BLOCKSZ * 4];
-  long bytes;
+  long chunk, bytes;
 
   while (1)
     {
-      bytes = read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      chunk = read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
 	{
-	  cpi_error = "error reading source file";
+	  ERROR(errno, "error reading source file");
 	  return -1;
 	}
-      else if (bytes == 0)
+      else if (chunk == 0)
 	break;
 
-      if (hfs_write(ofile, buf, bytes) < 0)
+      bytes = hfs_write(ofile, buf, chunk);
+      if (bytes == -1)
 	{
-	  cpi_error = hfs_error;
+	  ERROR(errno, hfs_error);
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
 	  return -1;
 	}
     }
@@ -269,10 +314,13 @@ int do_raw(int ifile, hfsfile *ofile)
  * DESCRIPTION:	open the source file; set hint for destination filename
  */
 static
-int opensrc(char *srcname, char **dsthint, char *ext)
+int opensrc(const char *srcname, const char **dsthint, const char *ext,
+	    int binary)
 {
   int fd, len;
-  static char name[HFS_MAX_FLEN + 1], *ptr;
+  static char name[HFS_MAX_FLEN + 1];
+  const char *cptr;
+  char *ptr;
 
   if (strcmp(srcname, "-") == 0)
     {
@@ -282,33 +330,33 @@ int opensrc(char *srcname, char **dsthint, char *ext)
   else
     fd = open(srcname, O_RDONLY);
 
-  if (fd < 0)
+  if (fd == -1)
     {
-      cpi_error = "error opening source file";
+      ERROR(errno, "error opening source file");
       return -1;
     }
 
-  ptr = strrchr(srcname, '/');
-  if (ptr == 0)
-    ptr = srcname;
+  cptr = strrchr(srcname, '/');
+  if (cptr == 0)
+    cptr = srcname;
   else
-    ++ptr;
+    ++cptr;
 
   if (ext == 0)
-    len = strlen(ptr);
+    len = strlen(cptr);
   else
     {
-      ext = strstr(ptr, ext);
+      ext = strstr(cptr, ext);
       if (ext == 0)
-	len = strlen(ptr);
+	len = strlen(cptr);
       else
-	len = ext - ptr;
+	len = ext - cptr;
     }
 
   if (len > HFS_MAX_FLEN)
     len = HFS_MAX_FLEN;
 
-  memcpy(name, ptr, len);
+  memcpy(name, cptr, len);
   name[len] = 0;
 
   for (ptr = name; *ptr; ++ptr)
@@ -335,21 +383,21 @@ int opensrc(char *srcname, char **dsthint, char *ext)
  * DESCRIPTION:	open the destination file
  */
 static
-hfsfile *opendst(hfsvol *vol, char *dstname, char *hint,
-		 char *type, char *creator)
+hfsfile *opendst(hfsvol *vol, const char *dstname, const char *hint,
+		 const char *type, const char *creator)
 {
   hfsdirent ent;
   hfsfile *file;
   long cwd;
 
-  if (hfs_stat(vol, dstname, &ent) >= 0 &&
+  if (hfs_stat(vol, dstname, &ent) != -1 &&
       (ent.flags & HFS_ISDIR))
     {
       cwd = hfs_getcwd(vol);
 
-      if (hfs_setcwd(vol, ent.cnid) < 0)
+      if (hfs_setcwd(vol, ent.cnid) == -1)
 	{
-	  cpi_error = hfs_error;
+	  ERROR(errno, hfs_error);
 	  return 0;
 	}
 
@@ -357,20 +405,11 @@ hfsfile *opendst(hfsvol *vol, char *dstname, char *hint,
     }
 
   hfs_delete(vol, dstname);
-  if (hfs_create(vol, dstname, type, creator) < 0)
-    {
-      cpi_error = hfs_error;
 
-      if (dstname == hint)
-	hfs_setcwd(vol, cwd);
-
-      return 0;
-    }
-
-  file = hfs_open(vol, dstname);
+  file = hfs_create(vol, dstname, type, creator);
   if (file == 0)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
 
       if (dstname == hint)
 	hfs_setcwd(vol, cwd);
@@ -380,9 +419,10 @@ hfsfile *opendst(hfsvol *vol, char *dstname, char *hint,
 
   if (dstname == hint)
     {
-      if (hfs_setcwd(vol, cwd) < 0)
+      if (hfs_setcwd(vol, cwd) == -1)
 	{
-	  cpi_error = hfs_error;
+	  ERROR(errno, hfs_error);
+
 	  hfs_close(file);
 	  return 0;
 	}
@@ -398,15 +438,15 @@ hfsfile *opendst(hfsvol *vol, char *dstname, char *hint,
 static
 void closefiles(int ifile, hfsfile *ofile, int *result)
 {
-  if (ofile && hfs_close(ofile) < 0 && *result == 0)
+  if (ofile && hfs_close(ofile) == -1 && *result == 0)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       *result = -1;
     }
 
-  if (close(ifile) < 0 && *result == 0)
+  if (close(ifile) == -1 && *result == 0)
     {
-      cpi_error = "error closing source file";
+      ERROR(errno, "error closing source file");
       *result = -1;
     }
 }
@@ -417,41 +457,44 @@ void closefiles(int ifile, hfsfile *ofile, int *result)
  * NAME:	cpi->macb()
  * DESCRIPTION:	copy a UNIX file to an HFS file using MacBinary II translation
  */
-int cpi_macb(char *srcname, hfsvol *vol, char *dstname)
+int cpi_macb(const char *srcname, hfsvol *vol, const char *dstname)
 {
   int ifile, result = 0;
   hfsfile *ofile;
   hfsdirent ent;
-  char *dsthint, type[5], creator[5];
+  const char *dsthint;
+  char type[5], creator[5];
   unsigned char buf[MACB_BLOCKSZ];
   unsigned short crc;
-  long dsize, rsize;
+  unsigned long dsize, rsize;
 
-  ifile = opensrc(srcname, &dsthint, ".bin");
-  if (ifile < 0)
+  ifile = opensrc(srcname, &dsthint, ".bin", 1);
+  if (ifile == -1)
     return -1;
 
   if (read(ifile, buf, MACB_BLOCKSZ) < MACB_BLOCKSZ)
     {
-      cpi_error = "error reading MacBinary file header";
+      ERROR(errno, "error reading MacBinary file header");
+
       close(ifile);
       return -1;
     }
 
   if (buf[0] != 0 || buf[74] != 0)
     {
-      cpi_error = "invalid MacBinary file header";
+      ERROR(EINVAL, "invalid MacBinary file header");
+
       close(ifile);
       return -1;
     }
 
-  crc = d_getw(&buf[124]);
+  crc = d_getuw(&buf[124]);
 
   if (crc_macb(buf, 124, 0x0000) != crc)
     {
       /* (buf[82] == 0) => MacBinary I? */
 
-      cpi_error = "unknown, unsupported, or corrupt MacBinary file";
+      ERROR(EINVAL, "unknown, unsupported, or corrupt MacBinary file");
 
       close(ifile);
       return -1;
@@ -459,7 +502,8 @@ int cpi_macb(char *srcname, hfsvol *vol, char *dstname)
 
   if (buf[123] > 129)
     {
-      cpi_error = "unsupported MacBinary file version";
+      ERROR(EINVAL, "unsupported MacBinary file version");
+
       close(ifile);
       return -1;
     }
@@ -467,18 +511,19 @@ int cpi_macb(char *srcname, hfsvol *vol, char *dstname)
   if (buf[1] < 1 || buf[1] > 63 ||
       buf[2 + buf[1]] != 0)
     {
-      cpi_error = "invalid MacBinary file header (bad file name)";
+      ERROR(EINVAL, "invalid MacBinary file header (bad file name)");
+
       close(ifile);
       return -1;
     }
 
-  dsize = d_getl(&buf[83]);
-  rsize = d_getl(&buf[87]);
+  dsize = d_getul(&buf[83]);
+  rsize = d_getul(&buf[87]);
 
-  if (dsize < 0 || dsize > 0x7fffffff ||
-      rsize < 0 || rsize > 0x7fffffff)
+  if (dsize > 0x7fffffff || rsize > 0x7fffffff)
     {
-      cpi_error = "invalid MacBinary file header (bad file length)";
+      ERROR(EINVAL, "invalid MacBinary file header (bad file length)");
+
       close(ifile);
       return -1;
     }
@@ -498,21 +543,21 @@ int cpi_macb(char *srcname, hfsvol *vol, char *dstname)
 
   result = do_macb(ifile, ofile, dsize, rsize);
 
-  if (result == 0 && hfs_fstat(ofile, &ent) < 0)
+  if (result == 0 && hfs_fstat(ofile, &ent) == -1)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       result = -1;
     }
 
   ent.fdflags = (buf[73] << 8 | buf[101]) &
     ~(HFS_FNDR_ISONDESK | HFS_FNDR_HASBEENINITED | HFS_FNDR_RESERVED);
 
-  ent.crdate = d_toutime(d_getl(&buf[91]));
-  ent.mddate = d_toutime(d_getl(&buf[95]));
+  ent.crdate = d_ltime(d_getul(&buf[91]));
+  ent.mddate = d_ltime(d_getul(&buf[95]));
 
-  if (result == 0 && hfs_fsetattr(ofile, &ent) < 0)
+  if (result == 0 && hfs_fsetattr(ofile, &ent) == -1)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       result = -1;
     }
 
@@ -527,14 +572,14 @@ int cpi_macb(char *srcname, hfsvol *vol, char *dstname)
  */
 static
 int binhx(char *fname, char *type, char *creator, short *fdflags,
-	  long *dsize, long *rsize)
+	  unsigned long *dsize, unsigned long *rsize)
 {
   int len;
   unsigned char byte, word[2], lword[4];
 
   if (bh_read(&byte, 1) < 1)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
@@ -542,19 +587,19 @@ int binhx(char *fname, char *type, char *creator, short *fdflags,
 
   if (len < 1 || len > HFS_MAX_FLEN)
     {
-      cpi_error = "invalid BinHex file header (bad file name)";
+      ERROR(EINVAL, "invalid BinHex file header (bad file name)");
       return -1;
     }
 
   if (bh_read(fname, len + 1) < len + 1)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
   if (fname[len] != 0)
     {
-      cpi_error = "invalid BinHex file header (bad file name)";
+      ERROR(EINVAL, "invalid BinHex file header (bad file name)");
       return -1;
     }
 
@@ -562,35 +607,34 @@ int binhx(char *fname, char *type, char *creator, short *fdflags,
       bh_read(creator, 4) < 4 ||
       bh_read(word, 2) < 2)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
-  *fdflags = d_getw(word);
+  *fdflags = d_getsw(word);
 
   if (bh_read(lword, 4) < 4)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
-  *dsize = d_getl(lword);
+  *dsize = d_getul(lword);
 
   if (bh_read(lword, 4) < 4)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
-  *rsize = d_getl(lword);
+  *rsize = d_getul(lword);
 
-  if (*dsize < 0 || *dsize > 0x7fffffff ||
-      *rsize < 0 || *rsize > 0x7fffffff)
+  if (*dsize > 0x7fffffff || *rsize > 0x7fffffff)
     {
-      cpi_error = "invalid BinHex file header (bad file length)";
+      ERROR(EINVAL, "invalid BinHex file header (bad file length)");
       return -1;
     }
 
-  if (bh_readcrc() < 0)
+  if (bh_readcrc() == -1)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
@@ -601,27 +645,29 @@ int binhx(char *fname, char *type, char *creator, short *fdflags,
  * NAME:	cpi->binh()
  * DESCRIPTION:	copy a UNIX file to an HFS file using BinHex translation
  */
-int cpi_binh(char *srcname, hfsvol *vol, char *dstname)
+int cpi_binh(const char *srcname, hfsvol *vol, const char *dstname)
 {
   int ifile, result;
   hfsfile *ofile;
   hfsdirent ent;
-  char *dsthint, fname[HFS_MAX_FLEN + 1], type[5], creator[5];
+  const char *dsthint;
+  char fname[HFS_MAX_FLEN + 1], type[5], creator[5];
   short fdflags;
-  long dsize, rsize;
+  unsigned long dsize, rsize;
 
-  ifile = opensrc(srcname, &dsthint, ".hqx");
-  if (ifile < 0)
+  ifile = opensrc(srcname, &dsthint, ".hqx", 0);
+  if (ifile == -1)
     return -1;
 
-  if (bh_open(ifile) < 0)
+  if (bh_open(ifile) == -1)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
+
       close(ifile);
       return -1;
     }
 
-  if (binhx(fname, type, creator, &fdflags, &dsize, &rsize) < 0)
+  if (binhx(fname, type, creator, &fdflags, &dsize, &rsize) == -1)
     {
       bh_close();
       close(ifile);
@@ -640,24 +686,24 @@ int cpi_binh(char *srcname, hfsvol *vol, char *dstname)
 
   result = do_binh(ofile, dsize, rsize);
 
-  if (bh_close() < 0 && result == 0)
+  if (bh_close() == -1 && result == 0)
     {
-      cpi_error = bh_error;
+      ERROR(errno, bh_error);
       result = -1;
     }
 
-  if (result == 0 && hfs_fstat(ofile, &ent) < 0)
+  if (result == 0 && hfs_fstat(ofile, &ent) == -1)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       result = -1;
     }
 
   ent.fdflags = fdflags &
     ~(HFS_FNDR_ISONDESK | HFS_FNDR_HASBEENINITED | HFS_FNDR_ISINVISIBLE);
 
-  if (result == 0 && hfs_fsetattr(ofile, &ent) < 0)
+  if (result == 0 && hfs_fsetattr(ofile, &ent) == -1)
     {
-      cpi_error = hfs_error;
+      ERROR(errno, hfs_error);
       result = -1;
     }
 
@@ -670,14 +716,14 @@ int cpi_binh(char *srcname, hfsvol *vol, char *dstname)
  * NAME:	cpi->text()
  * DESCRIPTION:	copy a UNIX file to an HFS file using text translation
  */
-int cpi_text(char *srcname, hfsvol *vol, char *dstname)
+int cpi_text(const char *srcname, hfsvol *vol, const char *dstname)
 {
   int ifile, result = 0;
   hfsfile *ofile;
-  char *dsthint;
+  const char *dsthint;
 
-  ifile = opensrc(srcname, &dsthint, ".txt");
-  if (ifile < 0)
+  ifile = opensrc(srcname, &dsthint, ".txt", 0);
+  if (ifile == -1)
     return -1;
 
   ofile = opendst(vol, dstname, dsthint, TEXT_TYPE, TEXT_CREA);
@@ -688,6 +734,7 @@ int cpi_text(char *srcname, hfsvol *vol, char *dstname)
     }
 
   result = do_text(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
@@ -697,14 +744,14 @@ int cpi_text(char *srcname, hfsvol *vol, char *dstname)
  * NAME:	cpi->raw()
  * DESCRIPTION:	copy a UNIX file to the data fork of an HFS file
  */
-int cpi_raw(char *srcname, hfsvol *vol, char *dstname)
+int cpi_raw(const char *srcname, hfsvol *vol, const char *dstname)
 {
   int ifile, result = 0;
   hfsfile *ofile;
-  char *dsthint;
+  const char *dsthint;
 
-  ifile = opensrc(srcname, &dsthint, 0);
-  if (ifile < 0)
+  ifile = opensrc(srcname, &dsthint, 0, 1);
+  if (ifile == -1)
     return -1;
 
   ofile = opendst(vol, dstname, dsthint, RAW_TYPE, RAW_CREA);
@@ -715,6 +762,7 @@ int cpi_raw(char *srcname, hfsvol *vol, char *dstname)
     }
 
   result = do_raw(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
