@@ -15,25 +15,111 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: copyout.c,v 1.13 1997/11/08 20:17:30 rob Exp $
  */
 
-# include <unistd.h>
-# include <sys/stat.h>
-# include <fcntl.h>
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_FCNTL_H
+#  include <fcntl.h>
+# else
+int open(const char *, int, ...);
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# else
+int dup(int);
+# endif
+
+# include <stdlib.h>
 # include <string.h>
+# include <errno.h>
+# include <sys/stat.h>
 
 # include "hfs.h"
 # include "data.h"
 # include "copyout.h"
+# include "charset.h"
 # include "binhex.h"
 # include "crc.h"
 
-char *cpo_error = "no error";
+const char *cpo_error = "no error";
+
+extern int errno;
+
+# define ERROR(code, str)	(cpo_error = (str), errno = (code))
 
 # define MACB_BLOCKSZ	128
 
 /* Copy Routines =========================================================== */
 
+/*
+ * NAME:	fork->macb()
+ * DESCRIPTION:	copy a single fork for MacBinary II
+ */
+static
+int fork_macb(hfsfile *ifile, int ofile, unsigned long size)
+{
+  char buf[HFS_BLOCKSZ * 4];
+  long chunk, bytes;
+  unsigned long total = 0;
+
+  while (1)
+    {
+      chunk = hfs_read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
+	{
+	  ERROR(errno, hfs_error);
+	  return -1;
+	}
+      else if (chunk == 0)
+	break;
+
+      bytes = write(ofile, buf, chunk);
+      if (bytes == -1)
+	{
+	  ERROR(errno, "error writing data");
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
+	  return -1;
+	}
+
+      total += bytes;
+    }
+
+  if (total != size)
+    {
+      ERROR(EIO, "inconsistent fork length");
+      return -1;
+    }
+
+  chunk = total % MACB_BLOCKSZ;
+  if (chunk)
+    {
+      memset(buf, 0, MACB_BLOCKSZ);
+      bytes = write(ofile, buf, MACB_BLOCKSZ - chunk);
+      if (bytes == -1)
+	{
+	  ERROR(errno, "error writing data");
+	  return -1;
+	}
+      else if (bytes != MACB_BLOCKSZ - chunk)
+	{
+	  ERROR(EIO, "wrong incomplete chunk");
+	  return -1;
+	}
+    }
+
+  return 0;
+}
+
 /*
  * NAME:	do_macb()
  * DESCRIPTION:	perform copy using MacBinary II translation
@@ -42,114 +128,110 @@ static
 int do_macb(hfsfile *ifile, int ofile)
 {
   hfsdirent ent;
-  unsigned char buf[HFS_BLOCKSZ * 4];
+  unsigned char buf[MACB_BLOCKSZ];
   long bytes;
-  unsigned long total;
 
-  if (hfs_fstat(ifile, &ent) < 0)
+  if (hfs_fstat(ifile, &ent) == -1)
     {
-      cpo_error = hfs_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  cpo_error = "error writing data";
-
   memset(buf, 0, MACB_BLOCKSZ);
 
   buf[1] = strlen(ent.name);
   strcpy((char *) &buf[2], ent.name);
 
-  memcpy(&buf[65], ent.type,    4);
-  memcpy(&buf[69], ent.creator, 4);
+  memcpy(&buf[65], ent.u.file.type,    4);
+  memcpy(&buf[69], ent.u.file.creator, 4);
 
   buf[73] = ent.fdflags >> 8;
 
-  d_putl(&buf[83], ent.dsize);
-  d_putl(&buf[87], ent.rsize);
+  d_putul(&buf[83], ent.u.file.dsize);
+  d_putul(&buf[87], ent.u.file.rsize);
 
-  d_putl(&buf[91], d_tomtime(ent.crdate));
-  d_putl(&buf[95], d_tomtime(ent.mddate));
+  d_putul(&buf[91], d_mtime(ent.crdate));
+  d_putul(&buf[95], d_mtime(ent.mddate));
 
   buf[101] = ent.fdflags & 0xff;
   buf[122] = buf[123] = 129;
 
-  d_putw(&buf[124], crc_macb(buf, 124, 0x0000));
+  d_putuw(&buf[124], crc_macb(buf, 124, 0x0000));
 
-  if (write(ofile, buf, MACB_BLOCKSZ) < MACB_BLOCKSZ)
+  bytes = write(ofile, buf, MACB_BLOCKSZ);
+  if (bytes == -1)
+    {
+      ERROR(errno, "error writing data");
+      return -1;
+    }
+  else if (bytes != MACB_BLOCKSZ)
+    {
+      ERROR(EIO, "wrote incomplete chunk");
+      return -1;
+    }
+
+  if (hfs_setfork(ifile, 0) == -1)
+    {
+      ERROR(errno, hfs_error);
+      return -1;
+    }
+
+  if (fork_macb(ifile, ofile, ent.u.file.dsize) == -1)
     return -1;
 
-  /* data fork */
+  if (hfs_setfork(ifile, 1) == -1)
+    {
+      ERROR(errno, hfs_error);
+      return -1;
+    }
+
+  if (fork_macb(ifile, ofile, ent.u.file.rsize) == -1)
+    return -1;
+
+  return 0;
+}
+
+/*
+ * NAME:	fork->binh()
+ * DESCRIPTION:	copy a single fork for BinHex
+ */
+static
+int fork_binh(hfsfile *ifile, unsigned long size)
+{
+  char buf[HFS_BLOCKSZ * 4];
+  long bytes;
+  unsigned long total = 0;
 
-  total = 0;
   while (1)
     {
       bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      if (bytes == -1)
 	{
-	  cpo_error = hfs_error;
+	  ERROR(errno, hfs_error);
 	  return -1;
 	}
       else if (bytes == 0)
 	break;
 
-      if (write(ofile, buf, bytes) < bytes)
-	return -1;
-
-      total += bytes;
-    }
-
-  if (total != ent.dsize)
-    {
-      cpo_error = "data fork length changed";
-      return -1;
-    }
-
-  bytes = total % MACB_BLOCKSZ;
-  if (bytes)
-    {
-      memset(buf, 0, MACB_BLOCKSZ);
-      if (write(ofile, buf, MACB_BLOCKSZ - bytes) < MACB_BLOCKSZ - bytes)
-	return -1;
-    }
-
-  /* resource fork */
-
-  if (hfs_setfork(ifile, 1) < 0)
-    {
-      cpo_error = hfs_error;
-      return -1;
-    }
-
-  total = 0;
-  while (1)
-    {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      if (bh_insert(buf, bytes) == -1)
 	{
-	  cpo_error = hfs_error;
+	  ERROR(errno, bh_error);
 	  return -1;
 	}
-      else if (bytes == 0)
-	break;
-
-      if (write(ofile, buf, bytes) < bytes)
-	return -1;
 
       total += bytes;
     }
 
-  if (total != ent.rsize)
+  if (total != size)
     {
-      cpo_error = "resource fork length changed";
+      ERROR(EIO, "inconsistent fork length");
       return -1;
     }
 
-  bytes = total % MACB_BLOCKSZ;
-  if (bytes)
+  if (bh_insertcrc() == -1)
     {
-      memset(buf, 0, MACB_BLOCKSZ);
-      if (write(ofile, buf, MACB_BLOCKSZ - bytes) < MACB_BLOCKSZ - bytes)
-	return -1;
+      ERROR(errno, bh_error);
+      return -1;
     }
 
   return 0;
@@ -163,125 +245,63 @@ static
 int binhx(hfsfile *ifile)
 {
   hfsdirent ent;
-  unsigned char byte, word[2], lword[4], buf[HFS_BLOCKSZ * 4];
-  long bytes;
-  unsigned long total;
+  unsigned char byte, word[2], lword[4];
 
-  if (hfs_fstat(ifile, &ent) < 0)
+  if (hfs_fstat(ifile, &ent) == -1)
     {
-      cpo_error = hfs_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  cpo_error = "error writing data";
-
   byte = strlen(ent.name);
-  if (bh_insert(&byte, 1) < 0 ||
-      bh_insert(ent.name, byte + 1) < 0 ||
-      bh_insert(ent.type, 4) < 0 ||
-      bh_insert(ent.creator, 4) < 0)
+  if (bh_insert(&byte, 1) == -1 ||
+      bh_insert(ent.name, byte + 1) == -1 ||
+      bh_insert(ent.u.file.type, 4) == -1 ||
+      bh_insert(ent.u.file.creator, 4) == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
-  d_putw(word, ent.fdflags);
-  if (bh_insert(word, 2) < 0)
+  d_putsw(word, ent.fdflags);
+  if (bh_insert(word, 2) == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
-  d_putl(lword, ent.dsize);
-  if (bh_insert(lword, 4) < 0)
+  d_putul(lword, ent.u.file.dsize);
+  if (bh_insert(lword, 4) == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
-  d_putl(lword, ent.rsize);
-  if (bh_insert(lword, 4) < 0 ||
-      bh_insertcrc() < 0)
+  d_putul(lword, ent.u.file.rsize);
+  if (bh_insert(lword, 4) == -1 ||
+      bh_insertcrc() == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
-  /* data fork */
-
-  total = 0;
-  while (1)
+  if (hfs_setfork(ifile, 0) == -1)
     {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
-	{
-	  cpo_error = hfs_error;
-	  return -1;
-	}
-      else if (bytes == 0)
-	break;
-
-      if (bh_insert(buf, bytes) < 0)
-	{
-	  cpo_error = bh_error;
-	  return -1;
-	}
-
-      total += bytes;
-    }
-
-  if (total != ent.dsize)
-    {
-      cpo_error = "data fork length changed";
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  if (bh_insertcrc() < 0)
+  if (fork_binh(ifile, ent.u.file.dsize) == -1)
+    return -1;
+
+  if (hfs_setfork(ifile, 1) == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, hfs_error);
       return -1;
     }
 
-  /* resource fork */
-
-  if (hfs_setfork(ifile, 1) < 0)
-    {
-      cpo_error = hfs_error;
-      return -1;
-    }
-
-  total = 0;
-  while (1)
-    {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
-	{
-	  cpo_error = hfs_error;
-	  return -1;
-	}
-      else if (bytes == 0)
-	break;
-
-      if (bh_insert(buf, bytes) < 0)
-	{
-	  cpo_error = bh_error;
-	  return -1;
-	}
-
-      total += bytes;
-    }
-
-  if (total != ent.rsize)
-    {
-      cpo_error = "resource fork length changed";
-      return -1;
-    }
-
-  if (bh_insertcrc() < 0)
-    {
-      cpo_error = bh_error;
-      return -1;
-    }
+  if (fork_binh(ifile, ent.u.file.rsize) == -1)
+    return -1;
 
   return 0;
 }
@@ -295,17 +315,17 @@ int do_binh(hfsfile *ifile, int ofile)
 {
   int result;
 
-  if (bh_start(ofile) < 0)
+  if (bh_start(ofile) == -1)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       return -1;
     }
 
   result = binhx(ifile);
 
-  if (bh_end() < 0 && result == 0)
+  if (bh_end() == -1 && result == 0)
     {
-      cpo_error = bh_error;
+      ERROR(errno, bh_error);
       result = -1;
     }
 
@@ -320,28 +340,45 @@ static
 int do_text(hfsfile *ifile, int ofile)
 {
   unsigned char buf[HFS_BLOCKSZ * 4], *ptr;
-  long bytes;
+  long chunk, bytes;
+  int len;
 
   while (1)
     {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      chunk = hfs_read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
 	{
-	  cpo_error = hfs_error;
+	  ERROR(errno, hfs_error);
 	  return -1;
 	}
-      else if (bytes == 0)
+      else if (chunk == 0)
 	break;
 
-      for (ptr = buf; ptr < buf + bytes; ++ptr)
+      for (ptr = buf; ptr < buf + chunk; ++ptr)
 	{
 	  if (*ptr == '\r')
 	    *ptr = '\n';
 	}
 
-      if (write(ofile, buf, bytes) < bytes)
+      len = chunk;
+      ptr = cs_latin1(buf, &len);
+      if (ptr == 0)
 	{
-	  cpo_error = "error writing data";
+	  ERROR(ENOMEM, 0);
+	  return -1;
+	}
+
+      bytes = write(ofile, ptr, len);
+      free(ptr);
+
+      if (bytes == -1)
+	{
+	  ERROR(errno, "error writing data");
+	  return -1;
+	}
+      else if (bytes != len)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
 	  return -1;
 	}
     }
@@ -356,23 +393,29 @@ int do_text(hfsfile *ifile, int ofile)
 static
 int do_raw(hfsfile *ifile, int ofile)
 {
-  unsigned char buf[HFS_BLOCKSZ * 4];
-  long bytes;
+  char buf[HFS_BLOCKSZ * 4];
+  long chunk, bytes;
 
   while (1)
     {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
+      chunk = hfs_read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
 	{
-	  cpo_error = hfs_error;
+	  ERROR(errno, hfs_error);
 	  return -1;
 	}
-      else if (bytes == 0)
+      else if (chunk == 0)
 	break;
 
-      if (write(ofile, buf, bytes) < bytes)
+      bytes = write(ofile, buf, chunk);
+      if (bytes == -1)
 	{
-	  cpo_error = "error writing data";
+	  ERROR(errno, "error writing data");
+	  return -1;
+	}
+      else if (bytes != chunk)
+	{
+	  ERROR(EIO, "wrote incomplete chunk");
 	  return -1;
 	}
     }
@@ -387,7 +430,8 @@ int do_raw(hfsfile *ifile, int ofile)
  * DESCRIPTION:	open the source file; set hint for destination filename
  */
 static
-hfsfile *opensrc(hfsvol *vol, char *srcname, char **dsthint, char *ext)
+hfsfile *opensrc(hfsvol *vol, const char *srcname,
+		 const char **dsthint, const char *ext)
 {
   hfsfile *file;
   hfsdirent ent;
@@ -396,14 +440,15 @@ hfsfile *opensrc(hfsvol *vol, char *srcname, char **dsthint, char *ext)
   file = hfs_open(vol, srcname);
   if (file == 0)
     {
-      cpo_error = hfs_error;
+      ERROR(errno, hfs_error);
       return 0;
     }
 
-  if (hfs_fstat(file, &ent) < 0)
+  if (hfs_fstat(file, &ent) == -1)
     {
+      ERROR(errno, hfs_error);
+
       hfs_close(file);
-      cpo_error = hfs_error;
       return 0;
     }
 
@@ -436,46 +481,44 @@ hfsfile *opensrc(hfsvol *vol, char *srcname, char **dsthint, char *ext)
  * DESCRIPTION:	open the destination file
  */
 static
-int opendst(char *dstname, char *hint)
+int opendst(const char *dstname, const char *hint)
 {
-  struct stat sbuf;
-  int fd, dirfd;
-
-  if (stat(dstname, &sbuf) >= 0 &&
-      S_ISDIR(sbuf.st_mode))
-    {
-      dirfd = open(".", O_RDONLY);
-      if (dirfd < 0)
-	{
-	  cpo_error = "error getting current directory";
-	  return -1;
-	}
-
-      if (chdir(dstname) < 0)
-	{
-	  cpo_error = "error setting current directory";
-	  close(dirfd);
-	  return -1;
-	}
-
-      dstname = hint;
-    }
+  int fd;
 
   if (strcmp(dstname, "-") == 0)
     fd = dup(STDOUT_FILENO);
   else
-    fd = open(dstname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
-
-  if (fd < 0)
     {
-      cpo_error = "error opening destination file";
-      fd = -1;
+      struct stat sbuf;
+      char *path = 0;
+
+      if (stat(dstname, &sbuf) != -1 &&
+	  S_ISDIR(sbuf.st_mode))
+	{
+	  path = malloc(strlen(dstname) + 1 + strlen(hint) + 1);
+	  if (path == 0)
+	    {
+	      ERROR(ENOMEM, 0);
+	      return -1;
+	    }
+
+	  strcpy(path, dstname);
+	  strcat(path, "/");
+	  strcat(path, hint);
+
+	  dstname = path;
+	}
+
+      fd = open(dstname, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+
+      if (path)
+	free(path);
     }
 
-  if (dstname == hint)
+  if (fd == -1)
     {
-      fchdir(dirfd);
-      close(dirfd);
+      ERROR(errno, "error opening destination file");
+      return -1;
     }
 
   return fd;
@@ -486,17 +529,17 @@ int opendst(char *dstname, char *hint)
  * DESCRIPTION:	open source and destination files
  */
 static
-int openfiles(hfsvol *vol, char *srcname, char *dstname, char *ext,
-	      hfsfile **ifile, int *ofile)
+int openfiles(hfsvol *vol, const char *srcname, const char *dstname,
+	      const char *ext, hfsfile **ifile, int *ofile)
 {
-  char *dsthint;
+  const char *dsthint;
 
   *ifile = opensrc(vol, srcname, &dsthint, ext);
   if (*ifile == 0)
     return -1;
 
   *ofile = opendst(dstname, dsthint);
-  if (*ofile < 0)
+  if (*ofile == -1)
     {
       hfs_close(*ifile);
       return -1;
@@ -512,15 +555,15 @@ int openfiles(hfsvol *vol, char *srcname, char *dstname, char *ext,
 static
 void closefiles(hfsfile *ifile, int ofile, int *result)
 {
-  if (close(ofile) < 0 && *result == 0)
+  if (close(ofile) == -1 && *result == 0)
     {
-      cpo_error = "error closing destination file";
+      ERROR(errno, "error closing destination file");
       *result = -1;
     }
 
-  if (hfs_close(ifile) < 0 && *result == 0)
+  if (hfs_close(ifile) == -1 && *result == 0)
     {
-      cpo_error = hfs_error;
+      ERROR(errno, hfs_error);
       *result = -1;
     }
 }
@@ -531,15 +574,16 @@ void closefiles(hfsfile *ifile, int ofile, int *result)
  * NAME:	cpo->macb()
  * DESCRIPTION:	copy an HFS file to a UNIX file using MacBinary II translation
  */
-int cpo_macb(hfsvol *vol, char *srcname, char *dstname)
+int cpo_macb(hfsvol *vol, const char *srcname, const char *dstname)
 {
   hfsfile *ifile;
   int ofile, result = 0;
 
-  if (openfiles(vol, srcname, dstname, ".bin", &ifile, &ofile) < 0)
+  if (openfiles(vol, srcname, dstname, ".bin", &ifile, &ofile) == -1)
     return -1;
 
   result = do_macb(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
@@ -549,15 +593,16 @@ int cpo_macb(hfsvol *vol, char *srcname, char *dstname)
  * NAME:	cpo->binh()
  * DESCRIPTION:	copy an HFS file to a UNIX file using BinHex translation
  */
-int cpo_binh(hfsvol *vol, char *srcname, char *dstname)
+int cpo_binh(hfsvol *vol, const char *srcname, const char *dstname)
 {
   hfsfile *ifile;
   int ofile, result;
 
-  if (openfiles(vol, srcname, dstname, ".hqx", &ifile, &ofile) < 0)
+  if (openfiles(vol, srcname, dstname, ".hqx", &ifile, &ofile) == -1)
     return -1;
 
   result = do_binh(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
@@ -567,15 +612,16 @@ int cpo_binh(hfsvol *vol, char *srcname, char *dstname)
  * NAME:	cpo->text()
  * DESCRIPTION:	copy an HFS file to a UNIX file using text translation
  */
-int cpo_text(hfsvol *vol, char *srcname, char *dstname)
+int cpo_text(hfsvol *vol, const char *srcname, const char *dstname)
 {
   hfsfile *ifile;
   int ofile, result = 0;
 
-  if (openfiles(vol, srcname, dstname, ".txt", &ifile, &ofile) < 0)
+  if (openfiles(vol, srcname, dstname, ".txt", &ifile, &ofile) == -1)
     return -1;
 
   result = do_text(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
@@ -585,15 +631,16 @@ int cpo_text(hfsvol *vol, char *srcname, char *dstname)
  * NAME:	cpo->raw()
  * DESCRIPTION:	copy the data fork of an HFS file to a UNIX file
  */
-int cpo_raw(hfsvol *vol, char *srcname, char *dstname)
+int cpo_raw(hfsvol *vol, const char *srcname, const char *dstname)
 {
   hfsfile *ifile;
   int ofile, result = 0;
 
-  if (openfiles(vol, srcname, dstname, 0, &ifile, &ofile) < 0)
+  if (openfiles(vol, srcname, dstname, 0, &ifile, &ofile) == -1)
     return -1;
 
   result = do_raw(ifile, ofile);
+
   closefiles(ifile, ofile, &result);
 
   return result;
