@@ -15,11 +15,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: copyout.h,v 1.4 1997/08/25 03:53:48 rob Exp $
  */
 
-extern char *cpo_error;
+extern const char *cpo_error;
 
-int cpo_macb(hfsvol *, char *, char *);
-int cpo_binh(hfsvol *, char *, char *);
-int cpo_text(hfsvol *, char *, char *);
-int cpo_raw(hfsvol *, char *, char *);
+typedef int (*cpofunc)(hfsvol *, const char *, const char *);
+
+int cpo_macb(hfsvol *, const char *, const char *);
+int cpo_binh(hfsvol *, const char *, const char *);
+int cpo_text(hfsvol *, const char *, const char *);
+int cpo_raw(hfsvol *, const char *, const char *);
