@@ -15,17 +15,21 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hattrib.c,v 1.9 1997/08/25 03:56:44 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
 # include <stdio.h>
-# include <string.h>
 # include <stdlib.h>
-# include <fcntl.h>
+# include <string.h>
 
 # include "hfs.h"
 # include "hcwd.h"
 # include "hfsutil.h"
-# include "glob.h"
 # include "hattrib.h"
 
 /*
@@ -48,7 +52,7 @@ int usage(void)
  */
 int hattrib_main(int argc, char *argv[])
 {
-  char *type = 0, *crea = 0;
+  const char *type = 0, *crea = 0;
   int invis = 0, lock = 0;
   hfsvol *vol;
   int fargc;
@@ -132,33 +136,32 @@ int hattrib_main(int argc, char *argv[])
       return 1;
     }
 
-  vol = hfs_remount(hcwd_getvol(-1), O_RDWR);
+  vol = hfsutil_remount(hcwd_getvol(-1), HFS_MODE_ANY);
   if (vol == 0)
     return 1;
 
-  fargv = hfs_glob(vol, argc - i, &argv[i], &fargc);
-  if (fargv == 0)
-    {
-      fprintf(stderr, "%s: globbing error\n", argv0);
-      result = 1;
-    }
-  else
+  fargv = hfsutil_glob(vol, argc - i, &argv[i], &fargc, &result);
+
+  if (result == 0)
     {
       for (i = 0; i < fargc; ++i)
 	{
 	  hfsdirent ent;
 
-	  if (hfs_stat(vol, fargv[i], &ent) < 0)
+	  if (hfs_stat(vol, fargv[i], &ent) == -1)
 	    {
-	      hfs_perrorp(fargv[i]);
+	      hfsutil_perrorp(fargv[i]);
 	      result = 1;
 	    }
 	  else
 	    {
-	      if (type)
-		memcpy(ent.type, type, 4);
-	      if (crea)
-		memcpy(ent.creator, crea, 4);
+	      if (! (ent.flags & HFS_ISDIR))
+		{
+		  if (type)
+		    memcpy(ent.u.file.type, type, 4);
+		  if (crea)
+		    memcpy(ent.u.file.creator, crea, 4);
+		}
 
 	      if (invis < 0)
 		ent.fdflags &= ~HFS_FNDR_ISINVISIBLE;
@@ -170,20 +173,16 @@ int hattrib_main(int argc, char *argv[])
 	      else if (lock > 0)
 		ent.flags |= HFS_ISLOCKED;
 
-	      if (hfs_setattr(vol, fargv[i], &ent) < 0)
+	      if (hfs_setattr(vol, fargv[i], &ent) == -1)
 		{
-		  hfs_perrorp(fargv[i]);
+		  hfsutil_perrorp(fargv[i]);
 		  result = 1;
 		}
 	    }
 	}
     }
 
-  if (hfs_umount(vol) < 0 && result == 0)
-    {
-      hfs_perror("Error closing HFS volume");
-      result = 1;
-    }
+  hfsutil_unmount(vol, &result);
 
   if (fargv)
     free(fargv);
