@@ -15,13 +15,22 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hcwd.c,v 1.8 1997/08/25 03:58:21 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# endif
+
 # include <stdio.h>
 # include <stdlib.h>
 # include <string.h>
 # include <errno.h>
-# include <unistd.h>
 
 # include "hfs.h"
 # include "hcwd.h"
@@ -34,6 +43,8 @@ static mountent *mounts = 0;
 static int mtabsz = 0, nmounts = 0;
 static int curvol = -1, dirty = 0;
 
+extern int errno;
+
 /*
  * NAME:	addent()
  * DESCRIPTION:	insert mount entry into table
@@ -67,8 +78,8 @@ int addent(mountent *ent)
  */
 int hcwd_init(void)
 {
-  char *home, *path;
-  char buf[512], *start, *ptr;
+  const char *home, *start;
+  char buf[512], *path, *ptr;
   mountent entry;
   int newcur = -1;
 
@@ -76,7 +87,7 @@ int hcwd_init(void)
   if (home == 0)
     home = "";
 
-  path = malloc(strlen(home) + 1 + strlen(STATEFNAME) + 1);
+  path = malloc(strlen(home) + 1 + sizeof(STATEFNAME));
   if (path == 0)
     return -1;
 
@@ -155,7 +166,7 @@ int hcwd_init(void)
 	  if (entry.cwd == 0)
 	    return -1;
 
-	  if (addent(&entry) < 0)
+	  if (addent(&entry) == -1)
 	    return -1;
 	}
     }
@@ -176,13 +187,13 @@ int hcwd_finish(void)
       int i;
 
       rewind(statef);
-      if (ftruncate(fileno(statef), 0) < 0)
+      if (ftruncate(fileno(statef), 0) == -1)
 	return -1;
 
       fprintf(statef, "%d\n", curvol);
 
       for (i = 0; i < nmounts; ++i)
-	fprintf(statef, "%s\t%lu\t%s\t%d\t%s\n",
+	fprintf(statef, "%s\t%ld\t%s\t%d\t%s\n",
 		mounts[i].vname,
 		mounts[i].vcrdate,
 		mounts[i].path,
@@ -207,7 +218,7 @@ int hcwd_finish(void)
       free(mounts);
     }
 
-  if (statef && fclose(statef) < 0)
+  if (statef && fclose(statef) == EOF)
     return -1;
 
   return 0;
@@ -217,7 +228,7 @@ int hcwd_finish(void)
  * NAME:	hcwd->mounted()
  * DESCRIPTION:	register a mounted volume
  */
-int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
+int hcwd_mounted(const char *vname, long vcrdate, const char *path, int partno)
 {
   mountent *entry, new;
 
@@ -254,7 +265,7 @@ int hcwd_mounted(char *vname, long vcrdate, char *path, int partno)
       new.cwd   == 0)
     return -1;
 
-  if (addent(&new) < 0)
+  if (addent(&new) == -1)
     return -1;
 
   curvol = nmounts - 1;
@@ -334,9 +345,10 @@ int hcwd_setvol(int vol)
  * NAME:	hcwd->setcwd()
  * DESCRIPTION:	change the current working directory for a mount entity
  */
-int hcwd_setcwd(mountent *ent, char *newcwd)
+int hcwd_setcwd(mountent *ent, const char *newcwd)
 {
-  char *path, *cwd;
+  const char *path;
+  char *cwd;
 
   path = newcwd;
   while (*path && *path != ':')
