@@ -17,8 +17,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
+# $Id: hfs.tcl,v 1.12 1997/11/05 05:27:13 rob Exp $
+#
 ###############################################################################
 
+proc ctime {secs} {
+    return [clock format $secs -format "%c"]
+}
+
 proc getvol {var} {
     global curvol
     upvar $var vol
@@ -39,15 +45,32 @@ proc hinfo {} {
 	set locked ""
     }
 
-    puts "Volume name is \"[$vol vname]\"$locked"
-    puts "Volume was created on [ctime [$vol crdate]]"
-    puts "Volume was last modified on [ctime [$vol mddate]]"
-    puts "Volume has [lindex [$vol size] 1] bytes free"
+    puts stdout "Volume name is \"[$vol vname]\"$locked"
+    puts stdout "Volume was created on [ctime [$vol crdate]]"
+    puts stdout "Volume was last modified on [ctime [$vol mddate]]"
+    puts stdout "Volume has [lindex [$vol size] 1] bytes free"
 }
 
-proc hmount {path {partno 1}} {
+proc hmount {path {partno ""}} {
     global mounts curpath curvol
 
+    set nparts [hfs nparts $path]
+    if {$nparts >= 0} {
+	puts stdout [concat "$path contains $nparts HFS"  \
+		[ternary {$nparts == 1} "partition" "partitions"]]
+    }
+
+    if {[string length $partno] == 0} {
+	if {$nparts > 0} {
+	    puts stderr "partition unspecified; selecting number 1"
+	    set partno 1
+	} elseif {$nparts == -1} {
+	    set partno 0
+	} else {
+	    set partno 1
+	}
+    }
+
     set vol [hfs mount $path $partno]
 
     if {[info exists mounts($path)]} {
@@ -100,22 +123,7 @@ proc hvol {name} {
 proc hpwd {} {
     getvol vol
 
-    set cwd [$vol cwd]
-    set path ""
-
-    while {$cwd != 1} {
-	set info [$vol dirinfo $cwd]
-
-	if {[info exists path]} {
-	    set path "[lindex $info 0]:$path"
-	} else {
-	    set path [lindex $info 0]
-	}
-
-	set cwd [lindex $info 1]
-    }
-
-    return $path
+    return "[join [$vol path] ":"]:"
 }
 
 proc hcd {{path ""}} {
@@ -174,14 +182,14 @@ proc hdir {{path ":"}} {
 	assoc $ent item
 
 	if {$item(kind) == "directory"} {
-	    puts [format "d%s %9lu item%s               %s %s"  \
+	    puts stdout [format "d%s %9lu item%s               %s %s"  \
 		    [ternary {[lsearch $item(flags) "invis"] >= 0} "i" " "]  \
 		    $item(size)  \
 		    [ternary {$item(size) == 1} " " "s"]  \
 		    [timestr $item(mddate)]  \
 		    $item(name)]
 	} else {
-	    puts [format "%s%s %4s/%4s %9lu %9lu %s %s"  \
+	    puts stdout [format "%s%s %4s/%4s %9lu %9lu %s %s"  \
 		    [ternary {[lsearch $item(flags) "locked"] >= 0} "F" "f"]  \
 		    [ternary {[lsearch $item(flags) "invis"] >= 0} "i" " "]  \
 		    $item(type)  \
@@ -212,7 +220,7 @@ proc hstat {{path ":"}} {
 	    set value $item($elt)
 	}
 
-	puts [format "%-10s %s" "$elt:" $value]
+	puts stdout [format "%-10s %s" "$elt:" $value]
     }
 }
 
@@ -235,7 +243,8 @@ proc hrmdir {args} {
 proc hcreate {path {type "TEXT"} {creator "UNIX"}} {
     getvol vol
 
-    $vol create $path $type $creator
+    set file [$vol create $path $type $creator]
+    $file close
 }
 
 proc htouch {args} {
@@ -288,7 +297,7 @@ proc hcat {path} {
 
 	regsub -all "\r" $buf "\n" buf
 
-	puts -nonewline $buf
+	puts -nonewline stdout $buf
     }
 }
 
@@ -345,15 +354,14 @@ proc hcopyin {path {dest ":"} {mode ""}} {
     $vol copyin $mode $path $dest
 }
 
-proc hformat {path {vname "Untitled"}} {
+proc hformat {path {partno 0} {vname "Untitled"} {badblocks {}}} {
     global mounts
 
     if {[info exists mounts($path)]} {
 	humount $path
     }
 
-    set partno 0
-    hfs format $path $partno $vname
+    hfs format $path $partno $vname $badblocks
 
     hmount $path $partno
 }
@@ -363,12 +371,12 @@ proc hformat {path {vname "Untitled"}} {
 proc help {{what ""}} {
     if {[string length $what]} {
 	if {[catch {info args h$what} msg]} {
-	    puts "Sorry, no help for \"$what\""
+	    puts stdout "Sorry, no help for \"$what\""
 	} else {
-	    puts "$what $msg"
+	    puts stdout "$what $msg"
 	}
     } else {
-	puts {
+	puts stdout {
 info		Show current volume information
 mount		Open a new HFS volume
 umount		Close an HFS volume
@@ -398,11 +406,11 @@ proc ? {args} {
 ###############################################################################
 
 proc license {} {
-    puts -nonewline "\n[hfs license]"
+    puts -nonewline stdout "\n[hfs license]"
 }
 
 proc version {} {
-    puts "[hfs version] - [hfs copyright]"
+    puts stdout "[hfs version] - [hfs copyright]"
 }
 
 if {[string compare [lindex $argv 0] "--license"] == 0} {
@@ -413,17 +421,20 @@ if {[string compare [lindex $argv 0] "--license"] == 0} {
 version
 
 if {[string compare [lindex $argv 0] "--version"] == 0} {
-    puts "`$argv0 --license' for licensing information."
+    puts stdout "`$argv0 --license' for licensing information."
     exit
 }
 
-puts "This is free software but comes with ABSOLUTELY NO WARRANTY."
-puts "Type `license' for details.\n"
+puts stdout "This is free software but comes with ABSOLUTELY NO WARRANTY."
+if {$hfs_interactive} {
+    puts stdout "Type `license' for details."
+}
+puts stdout ""
 
 ###############################################################################
 
 proc echo {args} {
-    puts [join $args " "]
+    puts stdout [join $args " "]
 }
 
 proc quit {} {
@@ -432,13 +443,21 @@ proc quit {} {
 
 ###############################################################################
 
+# Apparently some shells don't grok "$@" correctly
+if {$argc == 1 && [string length [lindex $argv 0]] == 0} {
+    incr argc -1
+    set argv [lreplace $argv 0 0]
+}
+
 if {$argc > 0} {
     eval hmount $argv
 }
 
 while {1} {
-    puts -nonewline "hfs> "
-    flush stdout
+    if {$hfs_interactive} {
+	puts -nonewline stdout "hfs> "
+	flush stdout
+    }
 
     if {[gets stdin line] == -1} {
 	exit
@@ -460,9 +479,12 @@ while {1} {
 
     if {[string length $msg] > 0} {
 	if {$result == 1} {
-	    puts "Error: $msg"
+	    puts stdout "Error: $msg"
+	    if {! $hfs_interactive} {
+		exit 1
+	    }
 	} else {
-	    puts $msg
+	    puts stdout $msg
 	}
     }
 }
