@@ -1,5 +1,5 @@
 /*
- * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * hfsck - tool for checking and repairing the integrity of HFS volumes
  * Copyright (C) 1996, 1997 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: ck_mdb.c,v 1.4 1997/11/11 23:16:54 rob Exp $
  */
 
 # include <stdio.h>
@@ -26,7 +28,7 @@
 # include "ck_mdb.h"
 
 /*
- * NAME:	ck_mdb()
+ * NAME:	ck->mdb()
  * DESCRIPTION:	verify/repair the master directory block
  */
 int ck_mdb(hfsvol *vol)
@@ -122,11 +124,12 @@ int ck_mdb(hfsvol *vol)
 
   /* mdb->drSigWord */
 
-  if (mdb->drSigWord != 0x4244 &&
-      ask("Bad volume signature (0x%04x); should be 0x4244", mdb->drSigWord))
+  if (mdb->drSigWord != HFS_SIGWORD &&
+      ask("Bad volume signature (0x%04x); should be 0x%04x",
+	  mdb->drSigWord, HFS_SIGWORD))
     {
-      mdb->drSigWord = 0x4244;
-      vol->flags |= HFS_UPDATE_MDB;
+      mdb->drSigWord = HFS_SIGWORD;
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
   /* mdb->drCrDate */
@@ -136,15 +139,15 @@ int ck_mdb(hfsvol *vol)
   if (mdb->drCrDate == 0 &&
       ask("Volume creation date is unset"))
     {
-      mdb->drCrDate = d_tomtime(now);
-      vol->flags |= HFS_UPDATE_MDB;
+      mdb->drCrDate = d_mtime(now);
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
-  if (d_toutime(mdb->drCrDate) > now &&
+  if (d_ltime(mdb->drCrDate) > now &&
       ask("Volume creation date is in the future (%s)", mctime(mdb->drCrDate)))
     {
-      mdb->drCrDate = d_tomtime(now);
-      vol->flags |= HFS_UPDATE_MDB;
+      mdb->drCrDate = d_mtime(now);
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
   /* mdb->drLsMod */
@@ -153,22 +156,22 @@ int ck_mdb(hfsvol *vol)
       ask("Volume last modify date is unset"))
     {
       mdb->drLsMod = mdb->drCrDate;
-      vol->flags |= HFS_UPDATE_MDB;
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
-  if (d_toutime(mdb->drLsMod) > now &&
+  if (d_ltime(mdb->drLsMod) > now &&
       ask("Volume last modify date is in the future (%s)",
 	  mctime(mdb->drLsMod)))
     {
-      mdb->drLsMod = d_tomtime(now);
-      vol->flags |= HFS_UPDATE_MDB;
+      mdb->drLsMod = d_mtime(now);
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
   if (mdb->drLsMod < mdb->drCrDate &&
       ask("Volume last modify date is before volume creation"))
     {
       mdb->drLsMod = mdb->drCrDate;
-      vol->flags |= HFS_UPDATE_MDB;
+      vol->flags |= HFS_VOL_UPDATE_MDB;
     }
 
   return 0;
