@@ -1,5 +1,5 @@
 /*
- * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * hfsck - tool for checking and repairing the integrity of HFS volumes
  * Copyright (C) 1996, 1997 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,11 +15,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hfsck.h,v 1.4 1997/09/14 19:58:48 rob Exp $
  */
 
-# include "internal.h"
+# include "libhfs.h"
 # include "data.h"
+# include "block.h"
 # include "low.h"
+# include "file.h"
+# include "btree.h"
+# include "node.h"
 # include "record.h"
 # include "volume.h"
 
