@@ -1,5 +1,5 @@
 /*
- * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * hfsck - tool for checking and repairing the integrity of HFS volumes
  * Copyright (C) 1996, 1997 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: main.c,v 1.9 1997/11/11 21:36:47 rob Exp $
  */
 
 # include <stdio.h>
@@ -55,19 +57,19 @@ int main(int argc, char *argv[])
   int pnum, result;
   hfsvol vol;
 
-  suid_init(0);
+  suid_init();
 
   if (argc == 2)
     {
       if (strcmp(argv[1], "--version") == 0)
 	{
-	  printf("%s - %s\n", VERSION, COPYRIGHT);
+	  printf("%s - %s\n", hfsutils_version, hfsutils_copyright);
 	  printf("`%s --license' for licensing information.\n", argv[0]);
 	  return 0;
 	}
       else if (strcmp(argv[1], "--license") == 0)
 	{
-	  printf("\n%s", LICENSE);
+	  printf("\n%s", hfsutils_license);
 	  return 0;
 	}
     }
@@ -107,67 +109,50 @@ int main(int argc, char *argv[])
   path = argv[optind];
   pnum = 1;
 
-  vol.flags  = REPAIR ? 0 : HFS_READONLY;
-  vol.pnum   = pnum;
-  vol.vstart = 0;
-  vol.vlen   = 0;
-  vol.lpa    = 0;
-  vol.vbm    = 0;
-  vol.cwd    = HFS_CNID_ROOTDIR;
+  v_init(&vol, 0);
 
-  vol.refs   = 0;
-  vol.files  = 0;
-  vol.dirs   = 0;
-  vol.prev   = 0;
-  vol.next   = 0;
-
-  vol.ext.map     = 0;
-  vol.ext.mapsz   = 0;
-  vol.ext.flags   = 0;
-  vol.ext.compare = r_compareextkeys;
-
-  vol.cat.map     = 0;
-  vol.cat.mapsz   = 0;
-  vol.cat.flags   = 0;
-  vol.cat.compare = r_comparecatkeys;
-
-  vol.fd = open(path, REPAIR ? O_RDWR : O_RDONLY);
-  if (vol.fd < 0 && REPAIR && (errno == EROFS || errno == EACCES))
+  if (REPAIR)
     {
-      fprintf(stderr, "%s: warning, %s not writable; cannot repair\n",
-	      argv[0], path);
-
-      options &= ~HFSCK_REPAIR;
-
-      vol.flags |= HFS_READONLY;
-      vol.fd = open(path, O_RDONLY);
+      result = v_open(&vol, path, HFS_MODE_RDWR);
+      if (result == -1)
+	{
+	  vol.flags |= HFS_VOL_READONLY;
+	  result = v_open(&vol, path, HFS_MODE_RDONLY);
+	}
     }
 
-  if (vol.fd < 0)
+  if (result == -1)
     {
       perror(path);
       return 1;
     }
 
-  if (l_lockvol(&vol) < 0 ||
-      l_readblock0(&vol) < 0 ||
-      l_readmdb(&vol) < 0)
+  if (REPAIR && (vol.flags & HFS_VOL_READONLY))
+    {
+      fprintf(stderr, "%s: warning: %s not writable; cannot repair\n",
+	      argv[0], path);
+
+      options &= ~HFSCK_REPAIR;
+    }
+
+  if (v_geometry(&vol, pnum) == -1 ||
+      v_readmdb(&vol) == -1)
     {
       perror(path);
-      close(vol.fd);
+      v_close(&vol);
       return 1;
     }
 
   result = hfsck(&vol);
 
-  if (result == 0 && v_flush(&vol, 1) < 0)
+  if (result == 0 && v_flush(&vol, 1) == -1)
     {
       perror("flushing volume");
-      close(vol.fd);
+      v_close(&vol);
       return 1;
     }
 
-  if (close(vol.fd) < 0)
+  if (v_close(&vol) == -1)
     {
       perror("closing volume");
       return 1;
