@@ -1,5 +1,5 @@
 /*
- * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * hfsck - tool for checking and repairing the integrity of HFS volumes
  * Copyright (C) 1996, 1997 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,6 +15,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: util.c,v 1.6 1997/11/11 23:17:05 rob Exp $
  */
 
 # include <stdio.h>
@@ -36,7 +38,7 @@ char *mctime(unsigned long secs)
   if (secs == 0)
     return "(Never)";
 
-  date = d_toutime(secs);
+  date = d_ltime(secs);
   strcpy(str, ctime(&date));
   str[24] = 0;
 
@@ -83,6 +85,18 @@ char *extstr(ExtDataRec *rec)
   return str;
 }
 
+/*
+ * NAME:	outhex()
+ * DESCRIPTION:	output a block of data in hex format
+ */
+void outhex(unsigned char *data, unsigned int len)
+{
+  int toggle = 0;
+
+  while (len--)
+    printf("%02x%s", *data++, toggle++ & 0x01 ? " " : "");
+}
+
 /*
  * NAME:	ask()
  * DESCRIPTION:	answer question from user
@@ -136,7 +150,7 @@ int ask(char *question, ...)
 	      break;
 	    }
 
-	  if (result < 0)
+	  if (result == -1)
 	    vprintf(question, args);
 	  else
 	    break;
