@@ -1,5 +1,5 @@
 /*
- * hfsutils - tools for reading and writing Macintosh HFS volumes
+ * hfsck - tool for checking and repairing the integrity of HFS volumes
  * Copyright (C) 1996, 1997 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,8 +15,11 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: util.h,v 1.4 1997/10/30 05:57:41 rob Exp $
  */
 
 char *mctime(unsigned long);
 char *extstr(ExtDataRec *);
+void outhex(unsigned char *, unsigned int);
 int ask(char *, ...);
