@@ -15,8 +15,18 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hfssh.c,v 1.7 1997/11/11 21:36:17 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# endif
+
 # include <tcl.h>
 
 # include "tclhfs.h"
@@ -28,6 +38,9 @@
  */
 int Tcl_AppInit(Tcl_Interp *interp)
 {
+  Tcl_SetVar(interp, "hfs_interactive",
+	     isatty(STDIN_FILENO) ? "1" : "0", TCL_GLOBAL_ONLY);
+
   if (Tcl_Init(interp) == TCL_ERROR)
     return TCL_ERROR;
 
@@ -43,7 +56,7 @@ int Tcl_AppInit(Tcl_Interp *interp)
  */
 int main(int argc, char *argv[])
 {
-  suid_init(0);
+  suid_init();
 
   Tcl_Main(argc, argv, Tcl_AppInit);
 
