@@ -15,19 +15,33 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hfsutil.c,v 1.12 1997/11/11 22:49:54 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
 # include <stdio.h>
 # include <stdlib.h>
 # include <string.h>
 # include <time.h>
-# include <errno.h>
 # include <ctype.h>
+# include <errno.h>
+# include <sys/stat.h>
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# else
+char *getcwd(char *buf, size_t size);
+# endif
 
 # include "hfs.h"
 # include "hcwd.h"
 # include "hfsutil.h"
 # include "suid.h"
+# include "glob.h"
 # include "version.h"
 
 # include "hattrib.h"
@@ -44,7 +58,7 @@
 # include "humount.h"
 # include "hvol.h"
 
-char *argv0, *bargv0;
+const char *argv0, *bargv0;
 
 /*
  * NAME:	main()
@@ -52,10 +66,11 @@ char *argv0, *bargv0;
  */
 int main(int argc, char *argv[])
 {
-  int i;
+  int i, len;
+  const char *dot;
 
   struct {
-    char *name;
+    const char *name;
     int (*func)(int, char *[]);
   } list[] = {
     { "hattrib", hattrib_main },
@@ -75,19 +90,19 @@ int main(int argc, char *argv[])
     { 0,         0            }
   };
 
-  suid_init(0);
+  suid_init();
 
   if (argc == 2)
     {
       if (strcmp(argv[1], "--version") == 0)
 	{
-	  printf("%s - %s\n", VERSION, COPYRIGHT);
+	  printf("%s - %s\n", hfsutils_version, hfsutils_copyright);
 	  printf("`%s --license' for licensing information.\n", argv[0]);
 	  return 0;
 	}
       else if (strcmp(argv[1], "--license") == 0)
 	{
-	  printf("\n%s", LICENSE);
+	  printf("\n%s", hfsutils_license);
 	  return 0;
 	}
     }
@@ -100,13 +115,18 @@ int main(int argc, char *argv[])
   else
     ++bargv0;
 
+  dot = strchr(bargv0, '.');
+  len = dot ? dot - bargv0 : strlen(bargv0);
+
   for (i = 0; list[i].name; ++i)
     {
-      if (strcmp(bargv0, list[i].name) == 0)
+      if (strncmp(bargv0, list[i].name, len) == 0)
 	{
 	  int result;
 
-	  if (hcwd_init() < 0)
+	  bargv0 = list[i].name;
+
+	  if (hcwd_init() == -1)
 	    {
 	      perror("Failed to initialize HFS working directories");
 	      return 1;
@@ -114,7 +134,7 @@ int main(int argc, char *argv[])
 
 	  result = list[i].func(argc, argv);
 
-	  if (hcwd_finish() < 0)
+	  if (hcwd_finish() == -1)
 	    {
 	      perror("Failed to save working directory state");
 	      return 1;
@@ -129,48 +149,38 @@ int main(int argc, char *argv[])
 }
 
 /*
- * NAME:	hfs->perror()
+ * NAME:	hfsutil->perror()
  * DESCRIPTION:	output an HFS error
  */
-void hfs_perror(char *msg)
+void hfsutil_perror(const char *msg)
 {
-  char *str = strerror(errno);
+  const char *str = strerror(errno);
 
   if (hfs_error == 0)
     fprintf(stderr, "%s: %s: %c%s\n", argv0, msg, tolower(*str), str + 1);
   else
-    {
-      if (errno)
-	fprintf(stderr, "%s: %s: %s (%s)\n", argv0, msg, hfs_error, str);
-      else
-	fprintf(stderr, "%s: %s: %s\n", argv0, msg, hfs_error);
-    }
+    fprintf(stderr, "%s: %s: %s (%s)\n", argv0, msg, hfs_error, str);
 }
 
 /*
- * NAME:	hfs_perrorp()
+ * NAME:	hfsutil->perrorp()
  * DESCRIPTION:	output an HFS error for a pathname
  */
-void hfs_perrorp(char *path)
+void hfsutil_perrorp(const char *path)
 {
-  char *str = strerror(errno);
+  const char *str = strerror(errno);
 
   if (hfs_error == 0)
     fprintf(stderr, "%s: \"%s\": %c%s\n", argv0, path, tolower(*str), str + 1);
   else
-    {
-      if (errno)
-	fprintf(stderr, "%s: \"%s\": %s (%s)\n", argv0, path, hfs_error, str);
-      else
-	fprintf(stderr, "%s: \"%s\": %s\n", argv0, path, hfs_error);
-    }
+    fprintf(stderr, "%s: \"%s\": %s (%s)\n", argv0, path, hfs_error, str);
 }
 
 /*
- * NAME:	hfs->remount()
+ * NAME:	hfsutil->remount()
  * DESCRIPTION:	mount a volume as though it were still mounted
  */
-hfsvol *hfs_remount(mountent *ment, int flags)
+hfsvol *hfsutil_remount(mountent *ment, int flags)
 {
   hfsvol *vol;
   hfsvolent vent;
@@ -188,7 +198,7 @@ hfsvol *hfs_remount(mountent *ment, int flags)
 
   if (vol == 0)
     {
-      hfs_perror(ment->path);
+      hfsutil_perror(ment->path);
       return 0;
     }
 
@@ -205,7 +215,7 @@ hfsvol *hfs_remount(mountent *ment, int flags)
       return 0;
     }
 
-  if (hfs_chdir(vol, ment->cwd) < 0)
+  if (hfs_chdir(vol, ment->cwd) == -1)
     {
       fprintf(stderr, "%s: Current HFS directory \"%s%s:\" no longer exists\n",
 	      argv0, ment->vname, ment->cwd);
@@ -215,10 +225,23 @@ hfsvol *hfs_remount(mountent *ment, int flags)
 }
 
 /*
- * NAME:	hfs->pinfo()
+ * NAME:	hfsutil->unmount()
+ * DESCRIPTION:	unmount a volume
+ */
+void hfsutil_unmount(hfsvol *vol, int *result)
+{
+  if (hfs_umount(vol) == -1 && *result == 0)
+    {
+      hfsutil_perror("Error closing HFS volume");
+      *result = 1;
+    }
+}
+
+/*
+ * NAME:	hfsutil->pinfo()
  * DESCRIPTION:	print information about a volume
  */
-void hfs_pinfo(hfsvolent *ent)
+void hfsutil_pinfo(hfsvolent *ent)
 {
   printf("Volume name is \"%s\"%s\n", ent->name,
 	 (ent->flags & HFS_ISLOCKED) ? " (locked)" : "");
@@ -228,10 +251,29 @@ void hfs_pinfo(hfsvolent *ent)
 }
 
 /*
- * NAME:	hfs->getcwd()
- * DESCRIPTION:	return full path to current directory
+ * NAME:	hfsutil->glob()
+ * DESCRIPTION:	perform filename globbing
  */
-char *hfs_cwd(hfsvol *vol)
+char **hfsutil_glob(hfsvol *vol, int argc, char *argv[],
+		    int *nelts, int *result)
+{
+  char **fargv;
+
+  fargv = hfs_glob(vol, argc, argv, nelts);
+  if (fargv == 0 && *result == 0)
+    {
+      fprintf(stderr, "%s: globbing error\n", argv0);
+      *result = 1;
+    }
+
+  return fargv;
+}
+
+/*
+ * NAME:	hfsutil->getcwd()
+ * DESCRIPTION:	return full path to current directory (must be free()'d)
+ */
+char *hfsutil_getcwd(hfsvol *vol)
 {
   char *path, name[33];
   long cwd;
@@ -247,7 +289,7 @@ char *hfs_cwd(hfsvol *vol)
       char *new;
       int namelen, i;
 
-      if (hfs_dirinfo(vol, &cwd, name) < 0)
+      if (hfs_dirinfo(vol, &cwd, name) == -1)
 	return 0;
 
       if (pathlen)
@@ -281,3 +323,80 @@ char *hfs_cwd(hfsvol *vol)
 
   return path;
 }
+
+/*
+ * NAME:	hfsutil->samepath()
+ * DESCRIPTION:	return 1 iff paths refer to same object
+ */
+int hfsutil_samepath(const char *path1, const char *path2)
+{
+  struct stat sbuf1, sbuf2;
+
+  return
+    stat(path1, &sbuf1) == 0 &&
+    stat(path2, &sbuf2) == 0 &&
+    sbuf1.st_dev == sbuf2.st_dev &&
+    sbuf1.st_ino == sbuf2.st_ino;
+}
+
+/*
+ * NAME:	hfsutil->abspath()
+ * DESCRIPTION:	make given UNIX path absolute (must be free()'d)
+ */
+char *hfsutil_abspath(const char *path)
+{
+  char *cwd, *buf;
+  size_t len;
+
+  if (path[0] == '/')
+    return strdup(path);
+
+  cwd = getenv("PWD");
+  if (cwd && hfsutil_samepath(cwd, "."))
+    {
+      buf = malloc(strlen(cwd) + 1 + strlen(path) + 1);
+      if (buf == 0)
+	return 0;
+
+      strcpy(buf, cwd);
+
+      goto finish;
+    }
+
+  len = 32;
+  cwd = malloc(len);
+  if (cwd == 0)
+    return 0;
+
+  while (getcwd(cwd, len) == 0)
+    {
+      if (errno != ERANGE)
+	{
+	  free(cwd);
+	  return 0;
+	}
+
+      len <<= 1;
+      buf = realloc(cwd, len);
+      if (buf == 0)
+	{
+	  free(cwd);
+	  return 0;
+	}
+
+      cwd = buf;
+    }
+
+  buf = realloc(cwd, strlen(cwd) + 1 + strlen(path) + 1);
+  if (buf == 0)
+    {
+      free(cwd);
+      return 0;
+    }
+
+finish:
+  strcat(buf, "/");
+  strcat(buf, path);
+
+  return buf;
+}
