@@ -15,10 +15,17 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hfswish.c,v 1.6 1997/11/11 21:36:09 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
 # include <stdio.h>
 # include <stdlib.h>
+
 # include <tk.h>
 
 # include "tclhfs.h"
@@ -26,7 +33,8 @@
 # include "suid.h"
 # include "images.h"
 
-static char *argv0;
+static
+char *argv0;
 
 # define DEFBITMAP(interp, name)  \
          Tk_DefineBitmap(interp, Tk_GetUid(#name),  \
@@ -103,7 +111,7 @@ int main(int argc, char *argv[])
   char **nargv;
   int i;
 
-  suid_init(0);
+  suid_init();
 
   argv0 = argv[0];
 
