@@ -15,17 +15,21 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: hrename.c,v 1.9 1997/08/25 04:00:46 rob Exp $
  */
 
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
 # include <stdio.h>
 # include <stdlib.h>
-# include <fcntl.h>
 # include <errno.h>
 
 # include "hfs.h"
 # include "hcwd.h"
 # include "hfsutil.h"
-# include "glob.h"
 # include "hrename.h"
 
 /*
@@ -33,25 +37,25 @@
  * DESCRIPTION:	move/rename files
  */
 static
-int do_rename(hfsvol *vol, int argc, char *argv[], char *dest)
+int do_rename(hfsvol *vol, int argc, char *argv[], const char *dest)
 {
   hfsdirent ent;
   int i, result = 0;
 
-  if (argc > 1 && (hfs_stat(vol, dest, &ent) < 0 ||
+  if (argc > 1 && (hfs_stat(vol, dest, &ent) == -1 ||
 		   ! (ent.flags & HFS_ISDIR)))
     {
       ERROR(ENOTDIR, 0);
-      hfs_perrorp(dest);
+      hfsutil_perrorp(dest);
 
       return 1;
     }
 
   for (i = 0; i < argc; ++i)
     {
-      if (hfs_rename(vol, argv[i], dest) < 0)
+      if (hfs_rename(vol, argv[i], dest) == -1)
 	{
-	  hfs_perrorp(argv[i]);
+	  hfsutil_perrorp(argv[i]);
 	  result = 1;
 	}
     }
@@ -77,30 +81,26 @@ int hrename_main(int argc, char *argv[])
       return 1;
     }
 
-  vol = hfs_remount(ment = hcwd_getvol(-1), O_RDWR);
+  vol = hfsutil_remount(ment = hcwd_getvol(-1), HFS_MODE_ANY);
   if (vol == 0)
     return 1;
 
-  fargv = hfs_glob(vol, argc - 2, &argv[1], &fargc);
-  if (fargv == 0)
-    {
-      fprintf(stderr, "%s: globbing error\n", argv0);
-      result = 1;
-    }
-  else
+  fargv = hfsutil_glob(vol, argc - 2, &argv[1], &fargc, &result);
+
+  if (result == 0)
     result = do_rename(vol, fargc, fargv, argv[argc - 1]);
 
   if (result == 0)
     {
       char *path;
 
-      path = hfs_cwd(vol);
+      path = hfsutil_getcwd(vol);
       if (path == 0)
 	{
-	  hfs_perror("Can't get current HFS directory path");
+	  hfsutil_perror("Can't get current HFS directory path");
 	  result = 1;
 	}
-      else if (hcwd_setcwd(ment, path) < 0)
+      else if (hcwd_setcwd(ment, path) == -1)
 	{
 	  perror("Can't set current HFS directory");
 	  result = 1;
@@ -110,11 +110,7 @@ int hrename_main(int argc, char *argv[])
 	free(path);
     }
 
-  if (hfs_umount(vol) < 0 && result == 0)
-    {
-      hfs_perror("Error closing HFS volume");
-      result = 1;
-    }
+  hfsutil_unmount(vol, &result);
 
   if (fargv)
     free(fargv);
