@@ -15,26 +15,43 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: suid.c,v 1.6 1997/11/11 21:35:55 rob Exp $
  */
 
-# include <unistd.h>
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# else
+uid_t getuid(void);
+uid_t geteuid(void);
+gid_t getgid(void);
+gid_t getegid(void);
+int setreuid(uid_t, uid_t);
+int setregid(gid_t, gid_t);
+# endif
 
 # include "suid.h"
 
-static uid_t euid;
-static gid_t egid;
+static uid_t uid, euid;
+static gid_t gid, egid;
 
 /*
  * NAME:	suid->init()
  * DESCRIPTION:	initialize application which may be running setuid/setgid
  */
-void suid_init(int enable)
+void suid_init(void)
 {
+  uid  = getuid();
+  gid  = getgid();
+
   euid = geteuid();
   egid = getegid();
 
-  if (! enable)
-    suid_disable();
+  suid_disable();
 }
 
 /*
@@ -43,8 +60,8 @@ void suid_init(int enable)
  */
 void suid_enable(void)
 {
-  seteuid(euid);
-  setegid(egid);
+  setreuid(-1, euid);
+  setregid(-1, egid);
 }
 
 /*
@@ -53,6 +70,6 @@ void suid_enable(void)
  */
 void suid_disable(void)
 {
-  seteuid(getuid());
-  setegid(getgid());
+  setreuid(-1, uid);
+  setregid(-1, gid);
 }
