@@ -15,16 +15,24 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * $Id: tclhfs.c,v 1.23 1997/11/11 21:30:42 rob Exp $
  */
 
-# include <unistd.h>
-# include <string.h>
-# include <stdlib.h>
+# ifdef HAVE_CONFIG_H
+#  include "config.h"
+# endif
+
+# ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+# endif
+
 # include <stdio.h>
-# include <time.h>
-# include <fcntl.h>
-# include <errno.h>
+# include <stdlib.h>
+# include <string.h>
 # include <ctype.h>
+# include <limits.h>
+
 # include <tcl.h>
 
 # include "tclhfs.h"
@@ -32,9 +40,12 @@
 # include "glob.h"
 # include "copyin.h"
 # include "copyout.h"
+# include "charset.h"
 # include "suid.h"
 # include "version.h"
 
+extern int errno;
+
 # define ERROR(code, str)	(hfs_error = (str), errno = (code))
 
 # define SIZE(type, n)		((size_t) (sizeof(type) * (n)))
@@ -47,7 +58,10 @@
 # define REALLOCX(ptr, type, n)  \
     ((n) ? REALLOC(type, n) : (FREE(ptr), (type *) 0))
 
-static int err_umount, err_close;
+# define CHARLEN(type)		((sizeof(type) * CHAR_BIT + 2) / 3 + 1)
+
+static
+int err_umount, err_close;
 
 typedef struct {
   hfsvol *vol;
@@ -68,9 +82,10 @@ Tcl_HashTable files;	/* mapping of frefs -> vrefs */
  * DESCRIPTION:	return a Tcl error for an HFS error
  */
 static
-int error(Tcl_Interp *interp, char *msg)
+int error(Tcl_Interp *interp, const char *msg)
 {
-  char *str, c[2];
+  const char *str;
+  char c[2];
 
   str  = strerror(errno);
   c[0] = tolower(*str);
@@ -84,12 +99,7 @@ int error(Tcl_Interp *interp, char *msg)
   if (hfs_error == 0)
     Tcl_AppendResult(interp, c, str + 1, (char *) 0);
   else
-    {
-      if (errno)
-	Tcl_AppendResult(interp, hfs_error, " (", str, ")", (char *) 0);
-      else
-	Tcl_AppendResult(interp, hfs_error, (char *) 0);
-    }
+    Tcl_AppendResult(interp, hfs_error, " (", str, ")", (char *) 0);
 
   return TCL_ERROR;
 }
@@ -101,23 +111,29 @@ int error(Tcl_Interp *interp, char *msg)
 static
 char *direntstr(hfsdirent *ent)
 {
-  char cnid[12], parid[12], rsize[12], dsize[12], crdate[12], mddate[12];
-  char *argv[22];
+  char
+    cnid[CHARLEN(unsigned long) + 1],
+    parid[CHARLEN(unsigned long) + 1],
+    rsize[CHARLEN(unsigned long) + 1],
+    dsize[CHARLEN(unsigned long) + 1],
+    crdate[CHARLEN(long) + 1],
+    mddate[CHARLEN(long) + 1],
+    bkdate[CHARLEN(long) + 1];
   register int argc;
+  char *argv[24];
   int locked, invis;
 
-  sprintf(cnid,   "%lu", ent->cnid);
-  sprintf(parid,  "%lu", ent->parid);
-  sprintf(rsize,  "%lu", ent->rsize);
-  sprintf(dsize,  "%lu", ent->dsize);
-  sprintf(crdate, "%lu", ent->crdate);
-  sprintf(mddate, "%lu", ent->mddate);
-
   argc = 0;
 
   argv[argc++] = "name";
   argv[argc++] = ent->name;
 
+  sprintf(cnid,   "%lu", ent->cnid);
+  sprintf(parid,  "%lu", ent->parid);
+
+  argv[argc++] = "cnid";
+  argv[argc++] = cnid;
+
   argv[argc++] = "parid";
   argv[argc++] = parid;
 
@@ -127,8 +143,7 @@ char *direntstr(hfsdirent *ent)
     {
       argv[argc++] = "directory";
 
-      argv[argc++] = "dirid";
-      argv[argc++] = cnid;
+      sprintf(dsize, "%u", ent->u.dir.valence);
 
       argv[argc++] = "size";
       argv[argc++] = dsize;
@@ -140,14 +155,11 @@ char *direntstr(hfsdirent *ent)
     {
       argv[argc++] = "file";
 
-      argv[argc++] = "fileid";
-      argv[argc++] = cnid;
-
       argv[argc++] = "type";
-      argv[argc++] = ent->type;
+      argv[argc++] = ent->u.file.type;
 
       argv[argc++] = "creator";
-      argv[argc++] = ent->creator;
+      argv[argc++] = ent->u.file.creator;
 
       locked = ent->flags & HFS_ISLOCKED;
       invis  = ent->fdflags & HFS_FNDR_ISINVISIBLE;
@@ -156,6 +168,9 @@ char *direntstr(hfsdirent *ent)
       argv[argc++] = (locked && invis) ? "locked invis" :
 	(locked ? "locked" : (invis ? "invis" : ""));
 
+      sprintf(rsize,  "%lu", ent->u.file.rsize);
+      sprintf(dsize,  "%lu", ent->u.file.dsize);
+
       argv[argc++] = "rsize";
       argv[argc++] = rsize;
 
@@ -163,12 +178,19 @@ char *direntstr(hfsdirent *ent)
       argv[argc++] = dsize;
     }
 
+  sprintf(crdate, "%ld", (long) ent->crdate);
+  sprintf(mddate, "%ld", (long) ent->mddate);
+  sprintf(bkdate, "%ld", (long) ent->bkdate);
+
   argv[argc++] = "crdate";
   argv[argc++] = crdate;
 
   argv[argc++] = "mddate";
   argv[argc++] = mddate;
 
+  argv[argc++] = "bkdate";
+  argv[argc++] = bkdate;
+
   return Tcl_Merge(argc, argv);
 }
 
@@ -177,15 +199,15 @@ char *direntstr(hfsdirent *ent)
  * DESCRIPTION:	collect and return the contents of an HFS directory
  */
 static
-int getdir(Tcl_Interp *interp, volref *vref, char *path)
+int getdir(Tcl_Interp *interp, volref *vref, const char *path)
 {
   hfsvol *vol = vref->vol;
   hfsdir *dir;
   hfsdirent ent;
   char *str;
 
-  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-      hfs_stat(vol, path, &ent) < 0)
+  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+      hfs_stat(vol, path, &ent) == -1)
     return error(interp, 0);
 
   if (ent.flags & HFS_ISDIR)
@@ -194,7 +216,7 @@ int getdir(Tcl_Interp *interp, volref *vref, char *path)
       if (dir == 0)
 	return error(interp, 0);
 
-      while (hfs_readdir(dir, &ent) >= 0)
+      while (hfs_readdir(dir, &ent) != -1)
 	{
 	  str = direntstr(&ent);
 	  if (str == 0)
@@ -209,7 +231,7 @@ int getdir(Tcl_Interp *interp, volref *vref, char *path)
 	  free(str);
 	}
 
-      if (hfs_closedir(dir) < 0)
+      if (hfs_closedir(dir) == -1)
 	return error(interp, 0);
     }
   else  /* ! HFS_ISDIR */
@@ -269,15 +291,15 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
       if (strcmp(argv[1], "close") == 0)
 	{
 	  Tcl_DeleteCommand(interp, argv[0]);
-	  if (err_close < 0)
+	  if (err_close == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "tell") == 0)
 	{
 	  long offset;
 
-	  offset = hfs_lseek(file, 0, SEEK_CUR);
-	  if (offset < 0)
+	  offset = hfs_seek(file, 0, HFS_SEEK_CUR);
+	  if (offset == -1)
 	    return error(interp, 0);
 
 	  sprintf(interp->result, "%lu", offset);
@@ -287,7 +309,7 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 	  hfsdirent ent;
 	  char *str;
 
-	  if (hfs_fstat(file, &ent) < 0)
+	  if (hfs_fstat(file, &ent) == -1)
 	    return error(interp, 0);
 
 	  str = direntstr(&ent);
@@ -337,8 +359,8 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 	  if (Tcl_ExprLong(interp, argv[2], &offset) != TCL_OK)
 	    return TCL_ERROR;
 
-	  offset = hfs_lseek(file, offset, SEEK_SET);
-	  if (offset < 0)
+	  offset = hfs_seek(file, offset, HFS_SEEK_SET);
+	  if (offset == -1)
 	    return error(interp, 0);
 
 	  sprintf(interp->result, "%lu", offset);
@@ -351,13 +373,13 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 	  if (Tcl_ExprLong(interp, argv[2], &bytes) != TCL_OK)
 	    return TCL_ERROR;
 
-	  if (bytes <= 0)
+	  if (bytes < 0)
 	    {
-	      interp->result = "size must be > 0";
+	      interp->result = "size must be >= 0";
 	      return TCL_ERROR;
 	    }
 
-	  mem = malloc(bytes + 1);
+	  mem = ALLOC(char, bytes + 1);
 	  if (mem == 0)
 	    {
 	      interp->result = "out of memory";
@@ -365,19 +387,25 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 	    }
 
 	  bytes = hfs_read(file, mem, bytes);
-	  if (bytes < 0)
+	  if (bytes == -1)
 	    {
 	      free(mem);
 	      return error(interp, 0);
 	    }
 
 	  mem[bytes] = 0;
+
 	  Tcl_SetResult(interp, mem, TCL_DYNAMIC);
 	}
       else if (strcmp(argv[1], "write") == 0)
 	{
-	  if (hfs_write(file, argv[2], strlen(argv[2])) < 0)
+	  long bytes;
+
+	  bytes = hfs_write(file, argv[2], strlen(argv[2]));
+	  if (bytes == -1)
 	    return error(interp, 0);
+
+	  sprintf(interp->result, "%lu", bytes);
 	}
       else
 	{
@@ -398,20 +426,20 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 
 	  if (strcmp(argv[3], "start") == 0 ||
 	      strcmp(argv[3], "set") == 0)
-	    whence = SEEK_SET;
+	    whence = HFS_SEEK_SET;
 	  else if (strcmp(argv[3], "current") == 0 ||
 		   strcmp(argv[3], "cur") == 0)
-	    whence = SEEK_CUR;
+	    whence = HFS_SEEK_CUR;
 	  else if (strcmp(argv[3], "end") == 0)
-	    whence = SEEK_END;
+	    whence = HFS_SEEK_END;
 	  else
 	    {
 	      interp->result = "bad arg 3: must be start, current, or end";
 	      return TCL_ERROR;
 	    }
 
-	  offset = hfs_lseek(file, offset, whence);
-	  if (offset < 0)
+	  offset = hfs_seek(file, offset, whence);
+	  if (offset == -1)
 	    return error(interp, 0);
 
 	  sprintf(interp->result, "%lu", offset);
@@ -433,6 +461,59 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
   return TCL_OK;
 }
 
+/*
+ * NAME:	fork->native()
+ * DESCRIPTION:	copy a single fork in native mode
+ */
+static
+int fork_native(Tcl_Interp *interp, hfsfile *ifile, hfsfile *ofile)
+{
+  char buf[HFS_BLOCKSZ * 8];
+  long chunk, bytes;
+
+  while (1)
+    {
+      chunk = hfs_read(ifile, buf, sizeof(buf));
+      if (chunk == -1)
+	return error(interp, "error reading source file");
+      else if (chunk == 0)
+	break;
+
+      bytes = hfs_write(ofile, buf, chunk);
+      if (bytes == -1)
+	return error(interp, "error writing destination file");
+      else if (bytes != chunk)
+	return error(interp, "wrote incomplete chunk");
+    }
+
+  return TCL_OK;
+}
+
+/*
+ * NAME:	file->ref()
+ * DESCRIPTION:	called to initialize a new file reference
+ */
+static
+void file_ref(Tcl_Interp *interp, volref *vref, fileref *fref, hfsfile *file)
+{
+  static int id = 0;
+  Tcl_CmdInfo info;
+  Tcl_HashEntry *entry;
+  int new;
+
+  do
+    sprintf(interp->result, "hfsfile%d", id++);
+  while (Tcl_GetCommandInfo(interp, interp->result, &info));
+
+  fref->file   = file;
+  fref->interp = interp;
+  fref->cmd    = Tcl_CreateCommand(interp, interp->result,
+				   file_cmd, fref, file_del);
+
+  entry = Tcl_CreateHashEntry(&files, (char *) fref, &new);
+  Tcl_SetHashValue(entry, vref);
+}
+
 /*
  * NAME:	do_copynative()
  * DESCRIPTION:	perform native HFS file copy
@@ -440,40 +521,23 @@ int file_cmd(ClientData clientData, Tcl_Interp *interp,
 static
 int do_copynative(Tcl_Interp *interp, hfsfile *ifile, hfsfile *ofile)
 {
-  char buf[HFS_BLOCKSZ * 8];
-  long bytes;
+  int result;
 
-  /* data fork */
+  if (hfs_setfork(ifile, 0) == -1 ||
+      hfs_setfork(ofile, 0) == -1)
+    return error(interp, "error opening data fork");
 
-  while (1)
-    {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
-	return error(interp, "error reading source file");
-      else if (bytes == 0)
-	break;
+  result = fork_native(interp, ifile, ofile);
+  if (result != TCL_OK)
+    return result;
 
-      if (hfs_write(ofile, buf, bytes) < bytes)
-	return error(interp, "error writing destination file");
-    }
-
-  /* resource fork */
-
-  if (hfs_setfork(ifile, 1) < 0 ||
-      hfs_setfork(ofile, 1) < 0)
+  if (hfs_setfork(ifile, 1) == -1 ||
+      hfs_setfork(ofile, 1) == -1)
     return error(interp, "error opening resource fork");
 
-  while (1)
-    {
-      bytes = hfs_read(ifile, buf, sizeof(buf));
-      if (bytes < 0)
-	return error(interp, "error reading source file");
-      else if (bytes == 0)
-	break;
-
-      if (hfs_write(ofile, buf, bytes) < bytes)
-	return error(interp, "error writing destination file");
-    }
+  result = fork_native(interp, ifile, ofile);
+  if (result != TCL_OK)
+    return result;
 
   return TCL_OK;
 }
@@ -489,7 +553,7 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
   Tcl_CmdInfo info;
   Tcl_HashEntry *entry = 0;
   hfsdirent ent;
-  char *srcname, *dstname;
+  const char *srcname, *dstname;
   hfsfile *ifile, *ofile;
   int result;
   long cnid;
@@ -508,14 +572,14 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
   srcname = argv[2];
   dstname = argv[4];
 
-  if (hfs_setcwd(srcvref->vol, srcvref->cwd) < 0)
+  if (hfs_setcwd(srcvref->vol, srcvref->cwd) == -1)
     return error(interp, 0);
 
   ifile = hfs_open(srcvref->vol, srcname);
   if (ifile == 0)
     return error(interp, "can't open source file");
 
-  if (hfs_setcwd(dstvref->vol, dstvref->cwd) < 0)
+  if (hfs_setcwd(dstvref->vol, dstvref->cwd) == -1)
     {
       error(interp, 0);
       hfs_close(ifile);
@@ -524,11 +588,11 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
 
   cnid = 0;
 
-  if (hfs_stat(dstvref->vol, dstname, &ent) >= 0)
+  if (hfs_stat(dstvref->vol, dstname, &ent) != -1)
     {
       if (ent.flags & HFS_ISDIR)
 	{
-	  if (hfs_setcwd(dstvref->vol, ent.cnid) < 0)
+	  if (hfs_setcwd(dstvref->vol, ent.cnid) == -1)
 	    {
 	      error(interp, 0);
 	      hfs_close(ifile);
@@ -537,14 +601,14 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
 
 	  dstname = srcname;
 
-	  if (hfs_stat(dstvref->vol, dstname, &ent) >= 0)
+	  if (hfs_stat(dstvref->vol, dstname, &ent) != -1)
 	    cnid = ent.cnid;
 	}
       else
 	cnid = ent.cnid;
     }
 
-  if (hfs_fstat(ifile, &ent) < 0)
+  if (hfs_fstat(ifile, &ent) == -1)
     {
       error(interp, "can't stat source file");
       hfs_close(ifile);
@@ -560,17 +624,12 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
     }
 
   hfs_delete(dstvref->vol, dstname);
-  if (hfs_create(dstvref->vol, dstname, ent.type, ent.creator) < 0)
-    {
-      error(interp, "can't create destination file");
-      hfs_close(ifile);
-      return TCL_ERROR;
-    }
 
-  ofile = hfs_open(dstvref->vol, dstname);
+  ofile = hfs_create(dstvref->vol, dstname,
+		     ent.u.file.type, ent.u.file.creator);
   if (ofile == 0)
     {
-      error(interp, "can't open destination file");
+      error(interp, "can't create destination file");
       hfs_close(ifile);
       return TCL_ERROR;
     }
@@ -579,13 +638,13 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
 
   ent.fdflags &= ~(HFS_FNDR_ISONDESK | HFS_FNDR_HASBEENINITED);
 
-  if (result == TCL_OK && hfs_fsetattr(ofile, &ent) < 0)
+  if (result == TCL_OK && hfs_fsetattr(ofile, &ent) == -1)
     result = error(interp, "can't set destination file attributes");
 
-  if (hfs_close(ofile) < 0 && result == TCL_OK)
+  if (hfs_close(ofile) == -1 && result == TCL_OK)
     result = error(interp, "error closing destination file");
 
-  if (hfs_close(ifile) < 0 && result == TCL_OK)
+  if (hfs_close(ifile) == -1 && result == TCL_OK)
     result = error(interp, "error closing source file");
 
   return result;
@@ -598,7 +657,7 @@ int copynative(Tcl_Interp *interp, volref *srcvref, char *argv[])
 static
 int copyin(Tcl_Interp *interp, hfsvol *vol, char *argv[])
 {
-  int (*copyfile)(char *, hfsvol *, char *);
+  cpifunc copyfile;
 
   if (strcmp(argv[2], "macbinary") == 0 ||
       strcmp(argv[2], "macb") == 0)
@@ -617,9 +676,9 @@ int copyin(Tcl_Interp *interp, hfsvol *vol, char *argv[])
       return TCL_ERROR;
     }
 
-  if (copyfile(argv[3], vol, argv[4]) < 0)
+  if (copyfile(argv[3], vol, argv[4]) == -1)
     {
-      ERROR(0, cpi_error);
+      ERROR(errno, cpi_error);
       return error(interp, 0);
     }
 
@@ -633,7 +692,7 @@ int copyin(Tcl_Interp *interp, hfsvol *vol, char *argv[])
 static
 int copyout(Tcl_Interp *interp, hfsvol *vol, char *argv[])
 {
-  int (*copyfile)(hfsvol *, char *, char *);
+  cpofunc copyfile;
 
   if (strcmp(argv[2], "macbinary") == 0 ||
       strcmp(argv[2], "macb") == 0)
@@ -652,9 +711,9 @@ int copyout(Tcl_Interp *interp, hfsvol *vol, char *argv[])
       return TCL_ERROR;
     }
 
-  if (copyfile(vol, argv[3], argv[4]) < 0)
+  if (copyfile(vol, argv[3], argv[4]) == -1)
     {
-      ERROR(0, cpo_error);
+      ERROR(errno, cpo_error);
       return error(interp, 0);
     }
 
@@ -662,16 +721,49 @@ int copyout(Tcl_Interp *interp, hfsvol *vol, char *argv[])
 }
 
 /*
- * NAME:	do_format()
- * DESCRIPTION:	call hfs_format() with necessary privileges
+ * NAME:	do_zero()
+ * DESCRIPTION:	call hfs_zero() with necessary privileges
  */
 static
-int do_format(char *path, int partno, char *vname)
+int do_zero(const char *path, int nparts, unsigned long *len)
 {
   int result;
 
   suid_enable();
-  result = hfs_format(path, partno, vname);
+  result = hfs_zero(path, nparts, len);
+  suid_disable();
+
+  return result;
+}
+
+/*
+ * NAME:	do_mkpart()
+ * DESCRIPTION:	call hfs_mkhfspart() with necessary privileges
+ */
+static
+int do_mkpart(const char *path, unsigned long len)
+{
+  int result;
+
+  suid_enable();
+  result = hfs_mkpart(path, len);
+  suid_disable();
+
+  return result;
+}
+
+/*
+ * NAME:	do_format()
+ * DESCRIPTION:	call hfs_format() with necessary privileges
+ */
+static
+int do_format(const char *path, int partno, const char *vname,
+	      int nbadblocks, unsigned long badblocks[])
+{
+  int result;
+
+  suid_enable();
+  result = hfs_format(path, partno, vname, nbadblocks, badblocks);
   suid_disable();
 
   return result;
@@ -752,14 +844,14 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	  hfsvolent ent;
 
 	  hfs_vstat(vol, &ent);
-	  sprintf(interp->result, "%lu", ent.crdate);
+	  sprintf(interp->result, "%ld", (long) ent.crdate);
 	}
       else if (strcmp(argv[1], "mddate") == 0)
 	{
 	  hfsvolent ent;
 
 	  hfs_vstat(vol, &ent);
-	  sprintf(interp->result, "%lu", ent.mddate);
+	  sprintf(interp->result, "%ld", (long) ent.mddate);
 	}
       else if (strcmp(argv[1], "islocked") == 0)
 	{
@@ -774,7 +866,7 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
       else if (strcmp(argv[1], "umount") == 0)
 	{
 	  Tcl_DeleteCommand(interp, argv[0]);
-	  if (err_umount < 0)
+	  if (err_umount == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "cwd") == 0)
@@ -790,7 +882,7 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	  id = vref->cwd;
 	  while (id != HFS_CNID_ROOTPAR)
 	    {
-	      if (hfs_dirinfo(vol, &id, name) < 0)
+	      if (hfs_dirinfo(vol, &id, name) == -1)
 		return error(interp, 0);
 
 	      Tcl_AppendElement(interp, name);
@@ -822,7 +914,7 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	}
       else if (strcmp(argv[1], "flush") == 0)
 	{
-	  if (hfs_flush(vol) < 0)
+	  if (hfs_flush(vol) == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "sepchar") == 0)
@@ -839,8 +931,8 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
       if (strcmp(argv[1], "cd") == 0 ||
 	  strcmp(argv[1], "chdir") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_chdir(vol, argv[2]) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_chdir(vol, argv[2]) == -1)
 	    return error(interp, 0);
 
 	  vref->cwd = hfs_getcwd(vol);
@@ -848,12 +940,12 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
       else if (strcmp(argv[1], "dirinfo") == 0)
 	{
 	  long id;
-	  char name[HFS_MAX_FLEN + 1], idstr[12];
+	  char name[HFS_MAX_FLEN + 1], idstr[CHARLEN(unsigned long) + 1];
 
 	  if (Tcl_ExprLong(interp, argv[2], &id) != TCL_OK)
 	    return TCL_ERROR;
 
-	  if (hfs_dirinfo(vol, &id, name) < 0)
+	  if (hfs_dirinfo(vol, &id, name) == -1)
 	    return error(interp, 0);
 
 	  sprintf(idstr, "%lu", id);
@@ -867,12 +959,8 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	}
       else if (strcmp(argv[1], "open") == 0)
 	{
-	  static int id = 0;
-	  hfsfile *file;
 	  fileref *fref;
-	  Tcl_CmdInfo info;
-	  Tcl_HashEntry *entry;
-	  int new;
+	  hfsfile *file;
 
 	  fref = ALLOC(fileref, 1);
 	  if (fref == 0)
@@ -881,33 +969,22 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	      return TCL_ERROR;
 	    }
 
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
 	      (file = hfs_open(vol, argv[2])) == 0)
 	    {
-	      error(interp, "can't open file");
 	      FREE(fref);
-	      return TCL_ERROR;
+	      return error(interp, 0);
 	    }
 
-	  do
-	    sprintf(interp->result, "hfsfile%d", id++);
-	  while (Tcl_GetCommandInfo(interp, interp->result, &info));
-
-	  fref->file   = file;
-	  fref->interp = interp;
-	  fref->cmd    = Tcl_CreateCommand(interp, interp->result,
-					   file_cmd, fref, file_del);
-
-	  entry = Tcl_CreateHashEntry(&files, (char *) fref, &new);
-	  Tcl_SetHashValue(entry, vref);
+	  file_ref(interp, vref, fref, file);
 	}
       else if (strcmp(argv[1], "stat") == 0)
 	{
 	  hfsdirent ent;
 	  char *str;
 
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_stat(vol, argv[2], &ent) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_stat(vol, argv[2], &ent) == -1)
 	    return error(interp, 0);
 
 	  str = direntstr(&ent);
@@ -921,28 +998,33 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	}
       else if (strcmp(argv[1], "mkdir") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_mkdir(vol, argv[2]) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_mkdir(vol, argv[2]) == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "rmdir") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_rmdir(vol, argv[2]) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_rmdir(vol, argv[2]) == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "delete") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_delete(vol, argv[2]) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_delete(vol, argv[2]) == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "touch") == 0)
 	{
 	  hfsdirent ent;
 
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_stat(vol, argv[2], &ent) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_stat(vol, argv[2], &ent) == -1)
+	    return error(interp, 0);
+
+	  ent.mddate = time(0);
+
+	  if (hfs_setattr(vol, argv[2], &ent) == -1)
 	    return error(interp, 0);
 	}
       else if (strcmp(argv[1], "glob") == 0)
@@ -950,7 +1032,7 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
 	  int listc, fargc;
 	  char **listv, **fargv, *result;
 
-	  if (hfs_setcwd(vol, vref->cwd) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1)
 	    return error(interp, 0);
 
 	  if (Tcl_SplitList(interp, argv[2], &listc, &listv) != TCL_OK)
@@ -981,8 +1063,8 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
     case 4:
       if (strcmp(argv[1], "rename") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_rename(vol, argv[2], argv[3]) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      hfs_rename(vol, argv[2], argv[3]) == -1)
 	    return error(interp, 0);
 	}
       else
@@ -996,22 +1078,37 @@ int vol_cmd(ClientData clientData, Tcl_Interp *interp,
     case 5:
       if (strcmp(argv[1], "create") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0 ||
-	      hfs_create(vol, argv[2], argv[3], argv[4]) < 0)
-	    return error(interp, 0);
+	  fileref *fref;
+	  hfsfile *file;
+
+	  fref = ALLOC(fileref, 1);
+	  if (fref == 0)
+	    {
+	      interp->result = "out of memory";
+	      return TCL_ERROR;
+	    }
+
+	  if (hfs_setcwd(vol, vref->cwd) == -1 ||
+	      (file = hfs_create(vol, argv[2], argv[3], argv[4])) == 0)
+	    {
+	      FREE(fref);
+	      return error(interp, 0);
+	    }
+
+	  file_ref(interp, vref, fref, file);
 	}
       else if (strcmp(argv[1], "copy") == 0)
 	return copynative(interp, vref, argv);
       else if (strcmp(argv[1], "copyin") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1)
 	    return error(interp, 0);
 
 	  return copyin(interp, vol, argv);
 	}
       else if (strcmp(argv[1], "copyout") == 0)
 	{
-	  if (hfs_setcwd(vol, vref->cwd) < 0)
+	  if (hfs_setcwd(vol, vref->cwd) == -1)
 	    return error(interp, 0);
 
 	  return copyout(interp, vol, argv);
@@ -1051,7 +1148,7 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 
   if (strcmp(argv[1], "mount") == 0)
     {
-      int partno = 1;
+      int partno;
       hfsvol *vol;
       volref *vref;
       Tcl_CmdInfo info;
@@ -1064,9 +1161,28 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	  return TCL_ERROR;
 	}
 
-      if (argc == 4 &&
-	  Tcl_GetInt(interp, argv[3], &partno) != TCL_OK)
-	return TCL_ERROR;
+      if (argc == 4)
+	{
+	  if (Tcl_GetInt(interp, argv[3], &partno) != TCL_OK)
+	    return TCL_ERROR;
+	}
+      else
+	{
+	  int nparts;
+
+	  nparts = hfs_nparts(argv[2]);
+
+	  if (nparts > 1)
+	    {
+	      sprintf(interp->result, "must specify partition number "
+		      "(%d available)", nparts);
+	      return TCL_ERROR;
+	    }
+	  else if (nparts == -1)
+	    partno = 0;
+	  else
+	    partno = 1;
+	}
 
       vref = ALLOC(volref, 1);
       if (vref == 0)
@@ -1076,7 +1192,7 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	}
 
       suid_enable();
-      vol = hfs_mount(argv[2], partno, O_RDWR);
+      vol = hfs_mount(argv[2], partno, HFS_MODE_ANY);
       suid_disable();
 
       if (vol == 0)
@@ -1098,11 +1214,56 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
       Tcl_CreateCommand(interp, interp->result,
 			vol_cmd, vref, vol_del);
     }
+  else if (strcmp(argv[1], "zero") == 0)
+    {
+      int nparts;
+      unsigned long len;
+
+      if (argc != 4)
+	{
+	  interp->result = "wrong # args";
+	  return TCL_ERROR;
+	}
+
+      if (Tcl_GetInt(interp, argv[3], &nparts) != TCL_OK)
+	return TCL_ERROR;
+
+      if (do_zero(argv[2], nparts, &len) == -1)
+	return error(interp, 0);
+
+      sprintf(interp->result, "%lu", len);
+    }
+  else if (strcmp(argv[1], "mkpart") == 0)
+    {
+      int len;
+
+      if (argc != 4)
+	{
+	  interp->result = "wrong # args";
+	  return TCL_ERROR;
+	}
+
+      if (Tcl_GetInt(interp, argv[3], &len) != TCL_OK)
+	return TCL_ERROR;
+
+      if (do_mkpart(argv[2], len) == -1)
+	return error(interp, 0);
+    }
+  else if (strcmp(argv[1], "nparts") == 0)
+    {
+      if (argc != 3)
+	{
+	  interp->result = "wrong # args";
+	  return TCL_ERROR;
+	}
+
+      sprintf(interp->result, "%d", hfs_nparts(argv[2]));
+    }
   else if (strcmp(argv[1], "format") == 0)
     {
-      int partno = 0;
+      int partno;
 
-      if (argc != 5)
+      if (argc < 5 || argc > 6)
 	{
 	  interp->result = "wrong # args";
 	  return TCL_ERROR;
@@ -1111,11 +1272,93 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
       if (Tcl_GetInt(interp, argv[3], &partno) != TCL_OK)
 	return TCL_ERROR;
 
-      if (do_format(argv[2], partno, argv[4]) < 0)
-	return error(interp, 0);
+      if (argc == 6)
+	{
+	  int listc, i;
+	  char **listv;
+	  unsigned long *badblocks;
+
+	  if (Tcl_SplitList(interp, argv[5], &listc, &listv) != TCL_OK)
+	    return TCL_ERROR;
+
+	  badblocks = ALLOCX(unsigned long, listc);
+	  if (listc && badblocks == 0)
+	    {
+	      free(listv);
+
+	      interp->result = "out of memory";
+	      return TCL_ERROR;
+	    }
+
+	  for (i = 0; i < listc; ++i)
+	    {
+	      if (Tcl_ExprLong(interp, listv[i], &badblocks[i]) != TCL_OK)
+		{
+		  free(listv);
+		  FREE(badblocks);
+		  return TCL_ERROR;
+		}
+	    }
+
+	  free(listv);
+
+	  if (do_format(argv[2], partno, argv[4], listc, badblocks) == -1)
+	    {
+	      FREE(badblocks);
+
+	      return error(interp, 0);
+	    }
+
+	  FREE(badblocks);
+	}
+      else
+	{
+	  if (do_format(argv[2], partno, argv[4], 0, 0) == -1)
+	    return error(interp, 0);
+	}
     }
   else if (strcmp(argv[1], "flushall") == 0)
     hfs_flushall();
+  else if (strcmp(argv[1], "chartrans") == 0)
+    {
+      char *result;
+
+      if (argc != 5)
+	{
+	  interp->result = "wrong # args";
+	  return TCL_ERROR;
+	}
+
+      if ((strcmp(argv[2], "latin1") != 0 &&
+	   strcmp(argv[2], "macroman") != 0) ||
+	  (strcmp(argv[3], "latin1") != 0 &&
+	   strcmp(argv[3], "macroman") != 0))
+	{
+	  interp->result = "bad arg to chartrans: "
+	    "charsets must be one of latin1, macroman";
+	  return TCL_ERROR;
+	}
+
+      if (strcmp(argv[2], "latin1") == 0 &&
+	  strcmp(argv[3], "macroman") == 0)
+	result = cs_macroman(argv[4], 0);
+      else if (strcmp(argv[2], "macroman") == 0 &&
+	       strcmp(argv[3], "latin1") == 0)
+	result = cs_latin1(argv[4], 0);
+      else
+	{
+	  Tcl_SetResult(interp, argv[4], TCL_VOLATILE);
+	  return TCL_OK;
+	}
+
+      if (result == 0)
+	{
+	  interp->result = "out of memory";
+	  return TCL_ERROR;
+	}
+
+      Tcl_SetResult(interp, result, TCL_DYNAMIC);
+    }
   else if (strcmp(argv[1], "version") == 0)
     {
       if (argc != 2)
@@ -1124,7 +1367,7 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	  return TCL_ERROR;
 	}
 
-      interp->result = VERSION;
+      interp->result = hfsutils_version;
     }
   else if (strcmp(argv[1], "copyright") == 0)
     {
@@ -1134,7 +1377,7 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	  return TCL_ERROR;
 	}
 
-      interp->result = COPYRIGHT;
+      interp->result = hfsutils_copyright;
     }
   else if (strcmp(argv[1], "author") == 0)
     {
@@ -1144,7 +1387,7 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	  return TCL_ERROR;
 	}
 
-      interp->result = AUTHOR;
+      interp->result = hfsutils_author;
     }
   else if (strcmp(argv[1], "license") == 0)
     {
@@ -1154,13 +1397,13 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
 	  return TCL_ERROR;
 	}
 
-      interp->result = LICENSE;
+      interp->result = hfsutils_license;
     }
   else
     {
       Tcl_AppendResult(interp, "bad hfs command \"", argv[1],
 		       "\": should be one of "
-		       "mount, format, flushall, "
+		       "mount, zero, mkpart, format, flushall, "
 		       "version, copyright, author, license",
 		       (char *) 0);
       return TCL_ERROR;
@@ -1169,36 +1412,6 @@ int cmd_hfs(ClientData clientData, Tcl_Interp *interp,
   return TCL_OK;
 }
 
-/*
- * NAME:	cmd->ctime()
- * DESCRIPTION:	implementation for ctime Tcl command
- */
-static
-int cmd_ctime(ClientData clientData, Tcl_Interp *interp,
-	      int argc, char *argv[])
-{
-  long ldate;
-  time_t date;
-  static char str[26];
-
-  if (argc != 2)
-    {
-      interp->result = "wrong # args";
-      return TCL_ERROR;
-    }
-
-  if (Tcl_ExprLong(interp, argv[1], &ldate) != TCL_OK)
-    return TCL_ERROR;
-
-  date = ldate;
-  strcpy(str, ctime(&date));
-  str[24] = 0;
-
-  interp->result = str;
-
-  return TCL_OK;
-}
-
 /*
  * NAME:	cmd->exit()
  * DESCRIPTION:	called to terminate; clean up volume state
@@ -1235,8 +1448,6 @@ int Hfs_Init(Tcl_Interp *interp)
   Tcl_InitHashTable(&files,   TCL_ONE_WORD_KEYS);
 
   Tcl_CreateCommand(interp, "hfs",   cmd_hfs,   0, 0);
-  Tcl_CreateCommand(interp, "ctime", cmd_ctime, 0, 0);
-
   Tcl_CreateCommand(interp, "exit",  cmd_exit,  0, 0);
 
   return TCL_OK;
