@@ -17,12 +17,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
+# $Id: xhfs.tcl,v 1.14 1997/11/11 23:03:46 rob Exp $
+#
 ###############################################################################
 
 set tk_strictMotif 1
 
 set gThinFont		{-*-helvetica-medium-r-normal-*-*-120-*-*-p-*-*-*}
-set gMessageFont	{-*-helvetica-medium-r-normal-*-*-100-*-*-p-*-*-*}
+set gSmallFont		{-*-helvetica-medium-r-normal-*-*-100-*-*-p-*-*-*}
 set gTypeFont		{-*-lucidatypewriter-medium-r-*-*-*-120-*-*-m-*-*-*}
 set gTitleFont		{-*-lucida-bold-i-normal-*-*-250-*-*-p-*-*-*}
 
@@ -52,6 +54,11 @@ if {[catch {exec uname} gUname] == 0} {
 		}
 	    }
 	}
+
+	FreeBSD {
+	    set gDefaultDevice /dev/rfd0
+	    set gLowFormatCmd [list fdformat $gDefaultDevice]
+	}
     }
 }
 
@@ -65,6 +72,29 @@ proc unique {} {
     return [incr gUnique]
 }
 
+proc window {title iconname} {
+    set w ".w[unique]"
+
+    toplevel $w -cursor left_ptr -relief raised -bd 1
+    wm title $w $title
+    wm iconname $w $iconname
+    wm transient $w .
+
+    wm withdraw $w
+
+    return $w
+}
+
+proc showwindow {w} {
+    update idletasks
+
+    set x [expr [winfo x .] + [winfo width  .]/2 - [winfo reqwidth  $w]/2]
+    set y [expr [winfo y .] + [winfo height .]/3 - [winfo reqheight $w]/3]
+
+    wm geom $w +$x+$y
+    wm deiconify $w
+}
+
 proc dialog {title cancel {ok ""} {default 0}} {
     set w ".d[unique]"
 
@@ -72,6 +102,8 @@ proc dialog {title cancel {ok ""} {default 0}} {
     wm title $w $title
     wm transient $w .
 
+    wm withdraw $w
+
     frame $w.top -relief raised -bd 1
     frame $w.bot -relief raised -bd 1
     pack $w.top -side top -expand 1 -fill both -ipadx 4m -ipady 4m
@@ -104,15 +136,7 @@ proc dialog {title cancel {ok ""} {default 0}} {
 }
 
 proc showdialog {w} {
-    wm withdraw $w
-    update idletasks
-
-    set x [expr [winfo x .] + [winfo width  .]/2 - [winfo reqwidth  $w]/2]
-    set y [expr [winfo y .] + [winfo height .]/3 - [winfo reqheight $w]/3]
-
-    wm geom $w +$x+$y
-    wm deiconify $w
-
+    showwindow $w
     grab set $w
 }
 
@@ -222,7 +246,7 @@ proc do_open {side} {
     if {[regexp "(.*)[$vol($side) sepchar]\$" $name ignore name]} {
 	mchdir $side $name
     } else {
-	show_file $vol($side) $name
+	show_file $side $name
     }
 }
 
@@ -508,7 +532,7 @@ proc format2 {w} {
 }
 
 proc help_about {} {
-    global gThinFont gTitleFont
+    global gThinFont gSmallFont gTitleFont
 
     set w [dialog "About" "OK"]
 
@@ -523,17 +547,17 @@ proc help_about {} {
 
     frame $f.space -height 2m
     label $f.vers -text [hfs version] -font $gThinFont
-    label $f.copy -text $copyright -font $gThinFont
-    label $f.email -text "E-Mail: [hfs author]" -font $gThinFont
+    label $f.copy -text $copyright -font $gSmallFont
+    label $f.auth -text [hfs author] -font $gSmallFont
 
-    pack $f.banner $f.space $f.vers $f.copy $f.email -side top
+    pack $f.banner $f.space $f.vers $f.copy $f.auth -side top
     pack $f -expand 1
 
     showdialog $w
 }
 
 proc help_license {} {
-    alert note "License" [fixtext [hfs license]] 190
+    alert note "Software License" [fixtext [hfs license]] 190
 }
 
 proc help_options {} {
@@ -599,7 +623,7 @@ proc ufs_automode {vol array} {
 	return macb
     } elseif {[regexp -nocase {\.hqx$} $name]} {
 	return binh
-    } elseif {[regexp -nocase {\.(txt|c|h)$} $name]} {
+    } elseif {[regexp -nocase {\.(txt|c|h|html?|rtf)$} $name]} {
 	return text
     } elseif {[regexp -nocase {\.(sit|sea|cpt|tar|gz|Z|gif|jpg)$} $name]} {
 	return raw
@@ -651,6 +675,12 @@ proc updatelist {side} {
 	return
     }
 
+    if {$fstype($side) == "hfs"} {
+	set chartrans {nulltrans}	;# {hfs charlocal}
+    } else {
+	set chartrans {nulltrans}
+    }
+
     set list [$vol($side) dir]
     set dir($side) {}
 
@@ -659,10 +689,10 @@ proc updatelist {side} {
     foreach ent $list {
 	assoc $ent item
 
+	set name [eval $chartrans [list $item(name)]]
+
 	if {$item(kind) == "directory"} {
-	    set name "$item(name)$sepchar"
-	} else {
-	    set name $item(name)
+	    set name "$name$sepchar"
 	}
 
 	if {$gShowAllFiles || [lsearch $item(flags) "invis"] < 0} {
@@ -802,12 +832,15 @@ proc close_volume {side} {
     updatelist $side; clearlists
 }
 
+proc ctime {secs} {
+    return [clock format $secs -format "%c"]
+}
+
 proc volume_info {side} {
     global gThinFont
     global vol dev fstype media
 
     set v $vol($side)
-    set w ".vinfo[unique]"
 
     if {$side == ".l"} {
 	set eside "Left"
@@ -815,10 +848,7 @@ proc volume_info {side} {
 	set eside "Right"
     }
 
-    toplevel $w -cursor left_ptr -relief raised -bd 1
-    wm title $w "$eside Info"
-    wm iconname $w "Info"
-    wm group $w .
+    set w [window "$eside Info" "Info"]
 
     set locked [$v islocked]
     set icon $media($side)
@@ -895,6 +925,8 @@ proc volume_info {side} {
     pack $f.md -side top -fill x
 
     pack $f -expand 1
+
+    showwindow $w
 }
 
 proc mmkdir {side} {
@@ -957,20 +989,28 @@ proc plural {count single plural} {
     }
 }
 
-proc show_file {vol fname} {
-    global gTypeFont
+proc nulltrans {str} {
+    return $str
+}
 
-    set w ".show[unique]"
+proc show_file {side fname} {
+    global gTypeFont
+    global vol fstype
 
     watch_cursor
 
-    # Make sure we can open the file before building the interface
-    set fh [$vol open $fname]
+    if {$fstype($side) == "hfs"} {
+	set charset macroman
+    } else {
+	set charset latin1
+    }
 
-    toplevel $w -cursor left_ptr -relief raised -bd 1
-    wm title $w $fname
-    wm iconname $w $fname
-    wm group $w .
+    # Make sure we can open the file before building the interface
+    set fh [$vol($side) open $fname]
+
+    set path [abspath [$vol($side) sepchar]  \
+	    [concat [$vol($side) path] [list $fname]] 1]
+    set w [window $path $fname]
 
     frame $w.x
     pack $w.x -expand 1 -fill both -padx 1m -pady 1m
@@ -986,7 +1026,7 @@ proc show_file {vol fname} {
     pack $f -fill both -expand 1
 
     watch_cursor $f.text xterm
-    after idle [list $f.text config -state disabled]
+    after idle "[list $f.text config -state disabled]; [list showwindow $w]"
 
     while {1} {
 	set buf [$fh read 512]
@@ -997,7 +1037,7 @@ proc show_file {vol fname} {
 
 	regsub -all "\r\n?" $buf "\n" buf
 
-	$f.text insert end $buf
+	$f.text insert end [hfs chartrans $charset latin1 $buf]
     }
 }
 
@@ -1040,23 +1080,39 @@ proc show_info {vol list} {
 
     assoc $list info
 
-    set w ".info[unique]"
-
-    toplevel $w -cursor left_ptr -relief raised -bd 1
-    wm title $w "$info(name) Info"
-    wm iconname $w "Info"
-    wm group $w .
-
     set isdir [expr [string compare $info(kind) "directory"] == 0]
     set ishfs [info exists info(crdate)]
 
+    if {$ishfs} {
+	set chartrans {nulltrans}	;# {hfs charlocal}
+    } else {
+	set chartrans {nulltrans}
+    }
+
+    set name [eval $chartrans [list $info(name)]]
+    set w [window "$name Info" $name]
+
+    set icon "document"
+
     if {$isdir} {
 	set icon "folder"
+
+	if {$ishfs} {
+	    set info(kind) "folder"
+	}
     } else {
-	if {$ishfs && [string compare $info(type) "APPL"] == 0} {
-	    set icon "application"
+	if {$ishfs} {
+	    if {[string compare $info(type) "APPL"] == 0} {
+		set icon "application"
+		set info(kind) "application"
+	    } else {
+		set info(kind) "document"
+	    }
 	} else {
-	    set icon "document"
+	    cd [$vol cwd]
+	    if {[file executable $info(name)]} {
+		set icon "application"
+	    }
 	}
     }
 
@@ -1064,14 +1120,7 @@ proc show_info {vol list} {
     wm iconmask $w ${icon}_mask
 
     if {$ishfs} {
-	set cwd [$vol cwd]
-	set where ""
-
-	while {$cwd != 1} {
-	    set dirinfo [$vol dirinfo $cwd]
-	    set where "[lindex $dirinfo 0]:$where"
-	    set cwd [lindex $dirinfo 1]
-	}
+	set where "[join [$vol path] ":"]:"
     } else {
 	set where [$vol cwd]
 
@@ -1094,7 +1143,7 @@ proc show_info {vol list} {
     frame $f.name
     frame $f.name.space -width 5
     label $f.name.icon -bitmap $icon
-    label $f.name.text -text $info(name) -font $gThinFont
+    label $f.name.text -text $name -font $gThinFont
     pack $f.name.space $f.name.icon $f.name.text -side left -padx 2m
 
     if {[lsearch $info(flags) "locked"] >= 0} {
@@ -1193,12 +1242,18 @@ proc show_info {vol list} {
     pack $f.md -side top -fill x
 
     pack $f -expand 1
+
+    showwindow $w
 }
 
-proc abspath {sepchar path} {
+proc abspath {sepchar path {asfile 0}} {
     switch $sepchar {
 	":" {
-	    return "[join $path ":"]:"
+	    if {$asfile} {
+		return [join $path ":"]
+	    } else {
+		return "[join $path ":"]:"
+	    }
 	}
 
 	"/" {
@@ -1405,8 +1460,12 @@ proc ufs_handle {dir} {
 		set s "\[ \t]+"
 		set d "\[0-9]+"
 
-		regexp "(.*)\n(\[^\n]*)\$" $info ignore head data
-		regexp "${s}($d)$s$d${s}($d)" $data ignore size avail
+		if {[catch {
+		    regexp "(.*)\n(\[^\n]*)\$" $info ignore head data
+		    regexp "${s}($d)$s$d${s}($d)" $data ignore size avail
+		}]} {
+		    return [list -1 -1]
+		}
 
 		set block 512
 		if {[regexp -nocase {kbyte|1024} $head]} {
@@ -1475,12 +1534,15 @@ proc mountdev {side device} {
 	set icon "folder"
     } else {
 	set size [lindex [$v size] 0]
-	set icon "harddisk"
 
 	if {$size <= [expr 1440 * 1024]} {
 	    set icon "floppy"
-	} elseif {$locked} {
+	} elseif {$locked &&  \
+		$size >= [expr 100 * 1024 * 1024] &&  \
+		$size <= [expr 800 * 1024 * 1024]} {
 	    set icon "cdrom"
+	} else {
+	    set icon "harddisk"
 	}
     }
 
@@ -1520,6 +1582,8 @@ proc flushtask {} {
     global gFlushInterval
 
     after $gFlushInterval flushtask
+
+    watch_cursor
     hfs flushall
 }
 
@@ -1532,13 +1596,13 @@ if {$argc > 0 && [string compare [lindex $argv 0] "--"] == 0} {
 }
 
 if {[string compare [lindex $argv 0] "--license"] == 0} {
-    puts -nonewline "\n[hfs license]"
+    puts -nonewline stdout "\n[hfs license]"
     exit
 }
 
 if {[string compare [lindex $argv 0] "--version"] == 0} {
-    puts "[hfs version] - [hfs copyright]"
-    puts "`$argv0 --license' for licensing information."
+    puts stdout "[hfs version] - [hfs copyright]"
+    puts stdout "`$argv0 --license' for licensing information."
     exit
 }
 
@@ -1547,7 +1611,7 @@ if {[string compare [lindex $argv 0] "--version"] == 0} {
 # Build the user interface
 
 proc makelist {w click other} {
-    global gMessageFont
+    global gSmallFont
 
     frame $w
 
@@ -1565,7 +1629,7 @@ proc makelist {w click other} {
     pack $w.info.x -expand 1
 
     label $w.info.x.icon
-    label $w.info.x.msg -font $gMessageFont -text ""
+    label $w.info.x.msg -font $gSmallFont -text ""
     label $w.info.x.lock -bitmap padlock
 
     pack $w.info.x.msg -side left -padx 0.5m
@@ -1737,8 +1801,8 @@ frame .m.2
 frame .m.2.ops -relief sunken -bd 1
 frame .m.2.ops.space1 -height 0.5m
 button .m.2.ops.info -state disabled -text "Get Info"
-button .m.2.ops.dele -state disabled -text "Delete"
 button .m.2.ops.rena -state disabled -text "Rename..."
+button .m.2.ops.dele -state disabled -text "Delete"
 frame .m.2.ops.space2 -height 0.5m
 
 pack  \
@@ -1836,11 +1900,11 @@ pack .l -side left -fill both -expand 1 -padx 3m
 pack .r -side right -fill both -expand 1 -padx 3m
 pack .m -side left
 
-wm title . "HFS Utility"
+wm title . "Macintosh HFS Volume Browser"
 
 wm iconbitmap . macdaemon
 wm iconmask . macdaemon_mask
-wm iconname . "xhfs"
+wm iconname . "HFS Browser"
 
 . config -cursor left_ptr
 
@@ -1854,7 +1918,7 @@ set fstype(.l) none
 set fstype(.r) none
 
 if {$argc > 2} {
-    puts "Usage: $argv0 \[left-vol \[right-vol]]"
+    puts stderr "Usage: $argv0 \[left-vol \[right-vol]]"
     exit 1
 }
 
